// Generated by MeshMaker (c) 2001 by Mychaeel <mychaeel@planetunreal.com>

class AttachRocket extends Projectile;

#exec obj load file=..\Textures\Starship.utx package=Starship
// Include EditPackages=BotPack in UnrealTournament.ini

#exec mesh import mesh=AttachRocket anivfile=Models\AttachRocket_a.3d datafile=Models\AttachRocket_d.3d x=0 y=0 z=0 mlod=0
#exec mesh origin mesh=AttachRocket x=0 y=0 z=0
#exec mesh sequence mesh=AttachRocket seq=All startframe=0 numframes=1

#exec meshmap new meshmap=AttachRocket mesh=AttachRocket
#exec meshmap scale meshmap=AttachRocket x=0.49676 y=0.49676 z=0.99353

var	Pawn	Target;
var	Pawn	Shooter;

var		Vector	LandSpot, LandNorm;
var		bool	bHitWater;
var()	float	AttachRotationSpeed;
var()	float	WallDistance;
var()	float	SeekAcceleration;
var()	float	SeekSpeed;
var()	float	SeekRadius;
var()	int		Bounces;
var()	float	AirRotateChange, AirRotateSpeed;
var()	float	GrenadeSpeedFactor;
var()	float	HomeSpeed;

var		Rotator	RotationDir;

simulated function ZoneChange( Zoneinfo NewZone )
{
	local waterring w;
	
	if (!NewZone.bWaterZone || bHitWater) Return;

	bHitWater = True;
	w = Spawn(class'WaterRing',,,,rot(16384,0,0));
	w.DrawScale = 0.2;
	w.RemoteRole = ROLE_None;
	Velocity = 0.6 * Velocity;
}

simulated event Bounce(Vector Normal)
{
	local Vector OldVel;

	OldVel = Velocity;
	Velocity = 0.75 * (( Velocity dot Normal ) * Normal * (-2.0) + Velocity);

	RSpin(0.9);
}

simulated function RSpin(float Force)
{
	SetRotation(Rotator(Vector(Rotation) + VRand() * Force));	
}

simulated function Rotator RSpinContinuous(float Force, float Randomness)
{
	local Rotator Result;

	if ( RotationDir == rot(0,0,0) )
		RotationDir = Rotator(VRand());

	Result = Rotator(Vector(Rotation) + (Vector(RotationDir) + VRand() * Randomness) * Force);
	SetRotation(Result);

	RotationDir = Rotation - Result;

	return RotationDir;
}

simulated event HomeAt(Actor Other)
{
	Velocity *= VSize(Velocity cross (EstimateLocation(Other) - Location)) * SeekAcceleration;
	Acceleration += (EstimateLocation(Other) - Location) * SeekSpeed;
	Acceleration /= 2;

	SetRotation(Rotator(Velocity + Acceleration * 0.2));
}

function PostBeginPlay()
{
	Super.PostBeginPlay();

	SetTimer(0.15, true);
	
	if ( Shooter == None )
		Shooter = Pawn(Owner);
		
	if ( Shooter == None )
	{
		if ( Weapon(Owner) != None )
		{
			Shooter = Pawn(Owner.Owner);

			if ( Shooter != None )
				return;
		}

		Log(name@"didn't find owner!");
	}
}

function Timer();

simulated function Landed(Vector Normal)
{
	HitWall(Normal, None);
}

simulated event bool PercentChance(float Percentage)
{
	return FRand() * 100 < Percentage;
}

simulated event Vector EstimateLocation(Actor Other)
{
	if ( Other == None )
		return Location;

	return Other.Location + (Other.Velocity / VSize(Velocity)) * 0.5;
}

simulated function NewExplode()
{
	Explode(Location, Vector(Rotation));
}

simulated function Explode(vector HitLocation, vector HitNormal)
{
	local AttachRocketBlast arb;

	arb = Spawn(class'AttachRocketBlast', self,, HitLocation, Rotator(HitNormal));
	arb.NumBlasts = 4;
	arb.DrawScale = DrawScale * 2;

	HurtRadius(250, 384, 'explosion', MomentumTransfer, Location);

	Destroy();
}

simulated event bool MonsterFriendly(ScriptedPawn Other)
{
	return Other == None || Other.AttitudeTo(Shooter) == ATTITUDE_Friendly;
}

simulated event bool IsEnemy(Pawn Other, Pawn FireAgent)
{
	return (
		Other != None
		&&
		(
			FireAgent == None
			^^
			(
				(
					ScriptedPawn(FireAgent) != None
					&& ScriptedPawn(FireAgent).AttitudeTo(Other) == ATTITUDE_Hate
				)
				||
				(
					FastTrace(Other.Location)
					&& Other != FireAgent
					&& !Other.IsInState('Dying')
					&&
					(
						!MonsterFriendly(ScriptedPawn(Other))
						|| !Level.Game.bTeamGame
						|| Other.PlayerReplicationInfo == None
						|| FireAgent.PlayerReplicationInfo == None
						|| Other.PlayerReplicationInfo.Team != FireAgent.PlayerReplicationInfo.Team
					)
				)
			)
		)
	);
}

simulated event Touch(Actor Other)
{
	if ( Pawn(Other) != None && IsEnemy(Pawn(Other), Shooter) )
		NewExplode();
}

simulated event HitWall(Vector Normal, Actor Wall);

auto state() FlyToAttach
{
	function BeginState()
	{
		SetPhysics(PHYS_Projectile);

		if ( Velocity == vect(0,0,0) )
			Velocity = Vector(Rotation) * speed;
	}

	simulated event Timer()
	{
		local Vector HLoc, HNorm;
		local bool Traced;
		
		Trace(HLoc, HNorm, Location + Vector(Rotation) * WallDistance);
		Traced = !FastTrace(Location + Vector(Rotation) * WallDistance);
		
		if ( Traced )
		{
			LandSpot = HLoc;
			LandNorm = HNorm;
			
			GoToState('FoundAttachSpot');
		}
	}

	simulated event HitWall(Vector Normal, Actor Wall)
	{
		SetRotation(Rotator(Normal));
	
		GoToState('Attached');
	}
}

state() GrenadeToAttach
{
	function BeginState()
	{
		SetPhysics(PHYS_Falling);

		if ( Velocity == vect(0,0,0) )
			Velocity = Vector(Rotation) * speed;

		Velocity *= GrenadeSpeedFactor;

		SetRotation(Rotator(VRand()));
		bFixedRotationDir = true;
	}

	simulated function Landed( vector HitNormal )
	{
		HitWall( HitNormal, None );
	}

	simulated event HitWall(Vector Normal, Actor Wall)
	{
		if ( Bounces > 0 )
		{
			Bounces--;
			Bounce(Normal);

			return;
		}

		SetRotation(Rotator(Normal));
	
		GoToState('Attached');
	}

	simulated function Tick(float TimeDelta)
	{
		RSpinContinuous(TimeDelta * AirRotateSpeed, TimeDelta * AirRotateChange);
	}
}

state FoundAttachSpot
{
	simulated event Timer()
	{
		if ( VSize(Location - LandSpot) < VSize(Velocity) / AttachRotationSpeed )
		{
			GoToState('Attaching');
		}
	}

	simulated event HitWall(Vector Normal, Actor Wall)
	{
		SetRotation(Rotator(Normal));
	
		GoToState('Attached');
	}
}

state Attaching
{
	simulated event Tick(float TimeDelta)
	{
		if ( VSize(Location - LandSpot) < CollisionHeight )
		{
			GotoState('Attached');
			
			return;
		}

		Log(VSize(Velocity));
		
		if ( Vector(Rotation) dot LandNorm == -1 || Vector(Rotation) dot LandNorm == 1 ) RSpin(0.1);
		else SetRotation(Rotator(Vector(Rotation) / (VSize(Location - LandSpot) / VSize(Velocity)) + (LandNorm * TimeDelta)));
	}
	
	simulated event HitWall(Vector Normal, Actor Wall)
	{
		SetRotation(Rotator(Normal));
	
		GoToState('Attached');
	}

	simulated function BeginState()
	{
		SetTimer(0, false);
	}
}

state Attached
{
	simulated event Timer()
	{
		local Pawn P;
	
		foreach RadiusActors(class'Pawn', P, SeekRadius)
			if ( IsEnemy(P, Shooter) )
			{
				Target = P;
				
				GoToState('Homing');
			}
	}

	simulated function BeginState()
	{
		SetPhysics(PHYS_None);
		bBlockPlayers = true;

		SetTimer(0.8, true);
	}
}

state Homing
{	
	simulated event Timer()
	{
		if ( !FastTrace(Target.Location) )
			GoToState('AfterHome');

		HomeAt(Target);
	}

	simulated function BeginState()
	{
		SetPhysics(PHYS_Projectile);

		speed = HomeSpeed;
		Velocity = Vector(Rotation) * HomeSpeed;
	}
	
	simulated event HitWall(Vector HitNormal, Actor Wall)
	{
		NewExplode();
	}

Begin:
	SetTimer(0.1, True);
	bBlockPlayers = false;
}

state AfterHome
{
	function BeginState()
	{
		SetTimer(0.15, true);

		SetPhysics(PHYS_Projectile);

		if ( Velocity == vect(0,0,0) )
			Velocity = Vector(Rotation) * speed;
	}

	simulated event Timer()
	{
		if ( FastTrace(Target.Location) )
			GoToState('Homing');
	}

	simulated event HitWall(Vector Normal, Actor Wall)
	{
		SetRotation(Rotator(Normal));
	
		GoToState('Attached');
	}	
}

defaultproperties
{
     AttachRotationSpeed=2.000000
     WallDistance=768.000000
     SeekAcceleration=3.000000
     SeekSpeed=7.000000
     SeekRadius=512.000000
     Bounces=3
     AirRotateChange=0.120000
     AirRotateSpeed=0.300000
     GrenadeSpeedFactor=4.000000
     HomeSpeed=1500.000000
     speed=350.000000
     MaxSpeed=600.000000
     Damage=30.000000
     MomentumTransfer=25000
     MyDamageType=RocketDeath
     SpawnSound=Sound'Botpack.SniperRifle.SniperFire'
     ImpactSound=Sound'UnrealShare.General.Expl04'
     ExplosionDecal=Class'Botpack.NuclearMark'
     LifeSpan=180.000000
     Mesh=Mesh'GusPackII.AttachRocket'
     DrawScale=0.180000
     bCollideWhenPlacing=True
     MultiSkins(0)=Texture'Starship.Base.Grey'
     MultiSkins(1)=Texture'Starship.Base.Rust'
     MultiSkins(2)=Texture'Starship.Base.d_gls'
     MultiSkins(3)=Texture'Starship.Base.C_bs1'
     MultiSkins(4)=Texture'Botpack.Ammocount.AmmoCountJunk'
     CollisionRadius=37.000000
     CollisionHeight=37.000000
     bBlockActors=True
     LightHue=28
     LightRadius=10
     bBounce=True
     Mass=250.000000
}
