//=============================================================================
// AutoRocket.
//=============================================================================
class AutoRocket expands UT_SeekingRocket;

var()	float			SeekAcceleration, SeekSpeed, TargetAcquiringRadius, WallAvoidSpeed, WallPredictionFactor, WallAvoidSlowdown, ShootDensity, ShootDistance, ShootDistanceFactor, MinHomePathDistance, MinWallAvoidNormal, TargetFindDensity, MaxPathDistance, FlockRadius, RotateRate, MinSeekAccel, SpaceFollowRadius, WallTurnDistance, WallReflection;
var()	bool			bShootAtTarget, bIgnoreIgnorers, bFlock;
var()	class<Actor>	ProjectileClass;
var		Pawn			User;
var		Actor			HomeTarget;
var		bool			bFoundUser;
var		int				Slew;


replication
{
	reliable if ( Role == ROLE_Authority )
		User,
		bFoundUser;
}

simulated function float DistanceTo(Vector Other)
{
	return VSize(Other - Location);
}

simulated function Vector SpaceNormal()
{
	local float x, y, z;
	local Vector v, cv, norm, s;

	for ( x = -1; x <= 1; x += 0.5 )
		for ( y = -1; y <= 1; y += 0.5 )
			for ( z = -1; z <= 1; z += 0.5 )
			{
				s.x = x;
				s.y = y;
				s.z = z;

				Trace(v, norm, Location + s * SpaceFollowRadius, Location);
				if ( v == vect(0,0,0) ) v = Location + s;
				
				cv += v - Location;
				
				// Log(self$"|"$s$"| ="@v@"at"@Location@"+++"$cv);
			}

	// Log(self@"#"@Location@"->"@cv);
	cv = Normal(cv);

	return cv;
}

simulated function float MinHomeNormal()
{
	return 0.1 * MinWallAvoidNormal;
}

simulated event bool PercentChance(float Percentage)
{
	return FRand() * 100 < Percentage;
}

simulated event Vector EstimateLocation(Actor Other)
{
	if ( Other == None )
		return Location;

	return Other.Location + (Other.Velocity / VSize(Velocity)) * 0.5;
}

simulated event HomeAt(Actor Other, optional bool bRotateOnly)
{
	local Vector Est;
	Est = EstimateLocation(Other);

	if ( bRotateOnly )
	{
		Velocity += Vector(Rotation) + (Est - Location) * RotateRate;
	}

	else
	{
		Acceleration += ((Est - Location) * SeekSpeed);
		Velocity *= (SeekAcceleration / (Velocity dot (Est - Location))) + MinSeekAccel;
	}
}

simulated event HomeToward(Vector HomeNormal, float Intensity, float Speeding)
{
	Velocity += Vector(Rotation) + Normal(HomeNormal) * RotateRate * Intensity;
	Acceleration += Normal(HomeNormal) * Speeding;
}

simulated event Actor WillHitWall(optional out Vector Location, optional out Vector OutNormal)
{
	if ( Seeking != None && FastTrace(Seeking.Location) )
		return none;

	return Trace(Location, OutNormal, Self.Location + Velocity * WallPredictionFactor);
}

simulated event bool CheckSeekable(Pawn Other, Pawn FireAgent)
{
	if ( FireAgent == None )
		return True;

	return (
		Other != None
		&& FastTrace(Other.Location)
		&& Other != FireAgent
		&& !Other.IsInState('Dying')
		&&
		(
			(
				ScriptedPawn(Other) == None
				|| ScriptedPawn(Other).AttitudeTo(User) != ATTITUDE_Friendly
			)
			&&
			(
				!Level.Game.bTeamGame
				|| Other.PlayerReplicationInfo == None
				|| FireAgent.PlayerReplicationInfo == None
				|| Other.PlayerReplicationInfo.Team != FireAgent.PlayerReplicationInfo.Team
			)
		)
	);
}

function BeginPlay()
{
	local Pawn P;

	if ( Instigator == None )
	{
		for ( P = Level.PawnList; P != None; P = P.NextPawn )
			if ( PlayerPawn(P) != None && (User == None || VSize(User.Location - Location) > VSize(P.Location - Location)) )
				User = P;
	}

	else
		User = Instigator;
}

simulated function SetTarget(Pawn Other)
{
	Seeking = Other;
}

simulated event Timer()
{
	local Pawn P, Target;
	local ut_spriteSmokePuff B;
	local Actor A;
	local AutoRocket AR;
	local Vector WallNorm, WallLoc;

	SetRotation(Rotator(Velocity));

	A = WillHitWall(WallLoc, WallNorm);

	if ( A != None && ( LevelInfo(A) != None || !CheckSeekable(Pawn(A), User) ) && ((VSize(WallLoc - Location) / VSize(Velocity * 0.1)) / 256) * MinHomeNormal() > (WallNorm dot Velocity) )
	{
		Velocity /= WallAvoidSlowdown;
		Acceleration -= (WallAvoidSpeed * 2 * (WallNorm dot Velocity) * (SpaceNormal() + WallNorm * WallReflection)) / (Min(WallPredictionFactor * VSize(Velocity), 256) / WallTurnDistance);
	}

	else if ( bFlock && Seeking == None )
	{
		foreach RadiusActors(class'AutoRocket', AR, FlockRadius)
		{
			HomeAt(AR, true);
		}
	}

	if ( User != None )
		bFoundUser = True;

	else if ( PercentChance(TargetFindDensity) )
	{
		HomeTarget = None;

		foreach RadiusActors(class'Pawn', P, TargetAcquiringRadius)
		{
			if ( Target == None || VSize(P.Location - Location) < VSize(Target.Location - Location) )
			{
				Target = P;
	
				if ( CheckSeekable(Target, User) )
					SetTarget(Target);
			}
		}
	}
}

defaultproperties
{
     SeekAcceleration=2.100000
     SeekSpeed=6.000000
     TargetAcquiringRadius=512.000000
     WallAvoidSpeed=3.500000
     WallPredictionFactor=0.425000
     WallAvoidSlowdown=1.300000
     ShootDistance=24.000000
     ShootDistanceFactor=1.000000
     MinHomePathDistance=256.000000
     MinWallAvoidNormal=1.000000
     TargetFindDensity=100.000000
     MaxPathDistance=384.000000
     FlockRadius=256.000000
     RotateRate=0.190000
     MinSeekAccel=0.700000
     SpaceFollowRadius=2048.000000
     WallTurnDistance=256.000000
     WallReflection=0.900000
     bShootAtTarget=True
     bIgnoreIgnorers=True
     bFlock=True
     ProjectileClass=Class'UnrealShare.TentacleProjectile'
     bCanTeleport=True
     bAlwaysRelevant=True
     bNetTemporary=False
     LifeSpan=250.000000
     bCollideWhenPlacing=True
     NetPriority=3.000000
     NetUpdateFrequency=150.000000
}
