//=============================================================================
// BioToxin.
//=============================================================================
class BioToxin expands UT_BioGel;

var()	float		WallTime, SizeFactor, ActorDamageRate, DissolveRate, AttachMomentumFactor, DripScale, UnstickRate, EvaporateAmount, EvaporateRate, EvaporateBias;
var()	bool		bNoStickIgnorers;

var		Actor		Attached;
var		bool		bDripping;
var		float		OrigScale;
var		AttachInfo	Attachment;

function Timer()
{
	DrawScale -= EvaporateAmount;

	if ( DrawScale < EvaporateBias )
		Destroy();
}

function HitWall(vector Normal, Actor Wall)
{
	Landed(Normal);
}

function Landed(vector Normal)
{
		
}

simulated function Grow(float Strenght)
{
	local float DS;
	local float NewRadius, NewHeight;

	DS = DrawScale;
	DrawScale += Strenght;

	NewRadius = (CollisionRadius / DS) * DrawScale;
	NewHeight = (CollisionHeight / DS) * DrawScale;

	SetCollisionSize(NewRadius, NewHeight);

	Log("Resized to DrawScale="$DrawScale@"CollisionRadius="$CollisionRadius@"CollisionHeight="$CollisionHeight);
}

simulated function Touch(Actor Other)
{	
	local BioToxin BT;

	if ( UT_BioGel(Other) != None )
	{
		Grow(Other.DrawScale);

		Other.Destroy();

		return;
	}

	if ( CanStick(Other) )
	{
		Attachment = class'AttachInfo'.static.CreateAttachment(Other, self, 16);
	
		Attached = Other;
		SurfaceNormal = Normal(Location - Other.Location);
		GoToState('OnActor');

		Super.Touch(Other);
	}

	else
		Super.Touch(Other);
}

function BioToxin HasBioglob(Actor Other)
{	
	local BioToxin BT;

	foreach AllActors(class'BioToxin', BT)
		if ( BT.Attached == Other || VSize(BT.Location - Other.Location) < 256 )
			return BT;

	return None;
}

function PostBeginPlay()
{
	local Actor A;

	SetTimer(1.0 / EvaporateRate, true);

	foreach TouchingActors(class'Actor', A)
		Touch(A);

	Super(Projectile).PostBeginPlay();
}

function PreBeginPlay()
{
	DrawScale *= SizeFactor;
	OrigScale = DrawScale;

	SetCollisionSize(CollisionRadius * SizeFactor, CollisionHeight * SizeFactor);
}

static function BioToxin SpreadBioglob(Actor Other, float Size, Biotoxin Source, out AttachInfo NewAttachment, optional bool bShareAttachment)
{
	local BioToxin BT;

	BT = Other.Spawn(Source.Class);

	if ( BT == None )
		return none;
	
	NewAttachment = class'AttachInfo'.static.CreateAttachment(Other, BT, 16);

	BT.DrawScale = Size;
	BT.Attached = Other;
	BT.SurfaceNormal = Normal(BT.Location - Other.Location);
	BT.GoToState('OnActor');
	BT.Instigator = Source.Instigator;

	if ( bShareAttachment )
		BT.Attachment = NewAttachment;

	return BT;
}

function bool CanStick(Actor Other)
{
	local bool r;

	r =
	(
		Instigator != None
		&& Other != None
		&&
		(
			Pawn(Other) == None
			^^
			(
				(
					TeamGamePlus(Level.Game) == None
					|| Pawn(Other).PlayerReplicationInfo == None
					|| Instigator.PlayerReplicationInfo == None
					|| Pawn(Other).PlayerReplicationInfo.Team != Instigator.PlayerReplicationInfo.Team
				)
				&&
				(
					ScriptedPawn(Other) == None
					||
					(
						ScriptedPawn(Other).AttitudeTo(Instigator) != ATTITUDE_Friendly
						&&
						(
							bNoStickIgnorers
							|| ScriptedPawn(Other).AttitudeTo(Instigator) != ATTITUDE_Ignore
						)
					)
				)
				&& Other != Instigator
			)
		)
	);

	Log(self@"can stick in"@Other$":"@r);

	return r;
}

state OnActor
{
	function TakeDamage(int Damage, Pawn Instigator, Vector HitLocation, vector Momentum, name DamageType);
	function Explode(Vector HitLocation, Vector HitNormal);

	simulated function Touch(Actor Other)
	{
		local AttachInfo A;
		local BioToxin B;

		if ( UT_BioGel(Other) != None )
		{
			DrawScale += UT_BioGel(Other).DrawScale;
			Other.Destroy();
		}

		else if ( Other != Attached && HasBioglob(Other) == None && CanStick(Other) )
		{
			DrawScale /= 2;

			B = class.static.SpreadBioglob(Other, DrawScale, self, A);
			B.Attachment = A;

			Log(self@"is spreading bioglob to:"@Other);

			Super.Touch(Other);
		}

		else
			Super.Touch(Other);
	}

	function EndState()
	{
		SetTimer(0.0, false);

		Velocity = vect(0,0,0);

		SetCollisionSize(CollisionRadius / 2, CollisionHeight / 2);
	}

	function BeginState()
	{
		SetPhysics(PHYS_Projectile);

		SetTimer(0.2, true);
	}

	function Timer()
	{
		local vector JumpSpeed;

		if ( Attached == None )
		{
			GoToState('Flying');

			return;
		}

		DrawScale += DissolveRate * 0.35;

		Attached.TakeDamage(ActorDamageRate * DrawScale * 0.2, Instigator, Location, vect(0,0,0), MyDamageType);

		if ( Pawn(Attached) != None && Pawn(Attached).Health <= 0 )
		{
			Log(self@"killed a pawn!");

			Pawn(Attached).KilledBy(Instigator);

			if ( Pawn(Attached).bIsPlayer )
			{
				GoToState('Flying');

				if ( Attachment != None )
					Attachment.Destroy();

				JumpSpeed.X = FRand() - 0.5 * 12;
				JumpSpeed.Y = FRand() - 0.5 * 12;
				JumpSpeed.Z = 32 + FRand() * 24;
	
				Velocity = JumpSpeed;
			}

			return;
		}
	}

Begin:
	SetCollisionSize(CollisionRadius * 2, CollisionHeight * 2);
}

static function BioToxin CreateBiotoxin(float Scale, Actor Anchor, optional Actor Attached, optional Vector Offset)
{
	local Biotoxin BT;

	if ( Anchor == None )
		return None;

	BT = Anchor.Spawn(default.Class, Anchor,, Anchor.Location + Offset);

	if ( Biotoxin(Anchor) != None )
		BT.Instigator = Biotoxin(Anchor).Instigator;

	if ( Attached != None )
	{
		BT.class.static.SpreadBioglob(Attached, BT.DrawScale, BT, BT.Attachment);

		BT.Destroy();
	}

	return BT;
}

simulated function Drip()
{
	local Biotoxin BT;

	BT = class.static.CreateBiotoxin(DrawScale * DripScale, self, None, vect(0,0,-80));
	BT.SetPhysics(PHYS_Falling);

	DrawScale -= DrawScale * DripScale;

	if ( DrawScale <= 0.5 * SizeFactor )
		GoToState('Exploding');
}

state OnSurface
{
	function Timer()
	{
		if ( DrawScale < 1.2 * UnstickRate * 0.05 )
		{
			GoToState('Exploding');
		
			return;
		}

		DrawScale -= UnstickRate * 0.05;
	}

	simulated function Touch(Actor Other)
	{
		if ( CanStick(Other) )
		{
			Class.static.SpreadBioglob(Other, DrawScale, self, Attachment, true);

			Destroy();
		}

		Super.Touch(Other);
	}

	simulated event AnimEnd()
	{
		Super.AnimEnd();

		if ( bDripping )
		{
			Drip();

			PlayAnim('Drip', 0.65, 0.8);
		}
	}

	simulated function CheckSurface()
	{
		Super.CheckSurface();

		if ( SurfaceNormal dot vect(0,0,-1) > 0.7 )
			bDripping = true;
	}

	function BeginPlay();

Begin:
	SetTimer(0.05, true);
}

auto state Flying {}

state Exploding
{
	function BeginState()
	{
		DrawScale = OrigScale;
		
		Super.BeginState();
	}
}

function TakeDamage(int Damage, Pawn Instigator, Vector HitLocation, vector Momentum, name DamageType);

defaultproperties
{
     wallTime=100.000000
     SizeFactor=2.000000
     ActorDamageRate=10.000000
     DissolveRate=1.450000
     AttachMomentumFactor=0.100000
     DripScale=0.210000
     UnstickRate=0.300000
     EvaporateAmount=0.020000
     EvaporateRate=15.000000
     EvaporateBias=0.100000
     LifeSpan=0.000000
     Texture=FireTexture'UnrealI.Effect26.fireeffect26'
}
