//=============================================================================
// EatingTentacle.
//=============================================================================
class EatingTentacle expands Tentacle;

var(Eating)	class<AttachInfo>	SnagType;
var(Eating)	float				EatInterval, EatRate, EatHealRate;
var			Carcass				Eaten;
var			AttachInfo			Snagging;
var			float				EatScale;
var			bool				bAcquireFood, bEating;
var			vector				Stomach;

function PostBeginPlay()
{
	bAcquireFood = true;

	Stomach = vect(0,0,-16);
	Stomach *= DrawScale;
}

function Actor ClosestActor(class<Actor> Type, Vector Loc, optional float Limit)
{
	local Actor A;
	local Actor R;

	if ( Limit <= 0 )
		foreach AllActors(Type, A)
		{
			if ( A != None && ( R == None || VSize(A.Location - Loc) < VSize(R.Location - Loc) ) )
				R = A;
	
			else if ( Limit > 0 )
				Log(VSize(A.Location - Loc)@Limit);
		}

	else
		foreach RadiusActors(Type, A, Limit)
			if ( A != None && ( R == None || VSize(A.Location - Loc) < VSize(R.Location - Loc) ) )
				R = A;
	
			else if ( Limit > 0 )
				Log(VSize(A.Location - Loc));

	return R;
}

function float Dist2D(Vector Other)
{
	local Vector V, N;

	V = Other;
	V.Z = 0;

	N = Location;
	N.Z = 0;

	return VSize(V - N);
}

state Waiting
{
KeepWaiting:
	NextAnim = '';

	Sleep(0.25);
	GoToState('Eating');
}

state Eating
{
	function EndState()
	{
		if ( Snagging != None )
			Snagging.Destroy();

		bEating = false;
	}

Begin:
	FinishAnim();
	bEating = true;

	if ( bAcquireFood )
	{
		Sleep(0.3);
	
		Eaten = Carcass(ClosestActor(class'Carcass', Location));
	}

	else
		bAcquireFood = true;

	if ( Eaten == None || Dist2D(Eaten.Location) > CollisionRadius + 64 )
	{
		Log(self@"has nothing to eat!");

		GoToState('Waiting');
	}

	else
	{
		Log(self@"acquired food:"@Eaten);

		EatScale = Eaten.DrawScale;

		GoTo('Eat');
	}

Eat:
	if ( Eaten == None || !bEating )
		GoToState('Waiting');

	else if ( VSize(Eaten.Location - (Location - Stomach)) > 24 )
		GoTo('Snag');

	else if ( Eaten.DrawScale <= 0.05 * EatScale )
		GoTo('Swallow');

	else
		GoTo('Chew');

Snag:
	Stomach = vect(0,0,-16);

	Log(self@"is snagging"@Eaten@"into"@Stomach + Location);

	while ( Snagging == None )
		Snagging = SnagType.static.CreateAttachment(self, Eaten, 0.6125, Stomach);

	Eaten.Lifespan = 0;
	Eaten.SetPhysics(PHYS_Projectile);
	Eaten.SetCollision(false, false, false);

	if ( VSize(Location - (Location - Stomach) ) > 16 )
		Stomach *= -1;

	while ( VSize(Eaten.Location - (Location - Stomach)) > 16 )
	{
		if ( Eaten == None )
		{
			Log(self$"'s food was destroyed unpristinely!");
	
			GoToState('Waiting');
		}

		Eaten.SetRotation(Rotator(Location - Eaten.Location));

		Sleep(0.3);
	}

	GoTo('Eat');

Chew:
	Log(self@"chews"@Eaten);

	DrawScale += EatRate;
	Eaten.DrawScale -= EatRate;

	Health += EatRate * EatHealRate;

	PlayAnim('Shoot', 1 / EatInterval, 0.25);

	FinishAnim();
	GoTo('Eat');

Swallow:
	Log(self@"swallowed"@Eaten);

	Eaten.Destroy();
	Snagging.RemoveAttachment();

	PlayAnim('Curl', 0.5 / EatInterval, 0.7);

	FinishAnim();

	NextAnim = '';
	
	LoopAnim('Move1', 0.1 / EatInterval, 0.3);

	Sleep(0.2);
	PlayAnim('');
	bEating = false;
	GoTo('Eat');
}


state Attacking
{
	ignores SeePlayer, HearNoise, Bump, HitWall;

	function ChooseAttackMode()
	{
		if (Physics == PHYS_Swimming)
		{
			Super.ChooseAttackMode();
			return;
		}
			
		if ((Enemy == None) || (Enemy.Health <= 0))
		{
			if ((OldEnemy != None) && (OldEnemy.Health > 0)) 
			{
				Enemy = OldEnemy;
				OldEnemy = None;
			}

			else
			{
				bAcquireFood = true;

				GotoState('Eating');
				return;
			}
		}
			
		if (!LineOfSightTo(Enemy))
			GotoState('StakeOut');

		else
			GotoState('RangedAttack');
	}
}

defaultproperties
{
     SnagType=Class'GusPackII.AttachInfo'
     EatInterval=1.400000
     EatRate=0.120000
     EatHealRate=120.000000
     WhipDamage=4
}
