//=============================================================================
// FriendMonster.
//=============================================================================
class FriendMonster expands CodeEvent;

var     ScriptedPawn    Closest;
var     Actor           FollowTarget;
var     Pawn            OldTarget;
var()   name            FollowState;
var()   name            FollowLabel;
var()   name            FollowedState;
var()   name            FollowedLabel;
var()   name            InitialPawnState;
var()   name            UninterruptableStates[32];
var()   float           FollowSkill;
var()   float           FollowSpeed;
var()   float           PullRunSpeed;
var()   float           CoverDistance;
var     bool            oc;
var     TranslocDest    TD;
var     FearSpot        FS;
var     float           PathInvWeight;
var     Weapon          OldWeap;
var     float           Care;
var()	bool			bAggressive;

var     FriendMonster   FM;
var     bool            bFollow;
var     float           TurnTime;
var     Actor           NextRunTarget;
var     vector          OldPawnVelocity;

function PostBeginPlay()
{
    local   ScriptedPawn    Current;
    local   Pawn            P;
    local   Vector          RL;
    local   Vector          RN;
    local   Actor           RA;
    local   FriendMonster   FP;
    local   bool            c;
    local   Actor           a;

    if ( Owner == None )
    {
        log("Warning:"@name@"has no master for Owner! Setting closest PlayerPawn/Bot instead.");

        for ( P = Level.PawnList; P != None; P = P.NextPawn )
            if ( Owner == None || ( PlayerPawn(P) != None || Bot(P) != None && VSize(P.Location - Location) < VSize(Owner.Location - Location) && P != None ) )
                SetOwner(P);
                
        if ( Owner == None )
            foreach AllActors(class'Pawn', P)
                SetOwner(P);
    }

    else if ( Bot(Owner) == None && PlayerPawn(Owner) == None )
    {
        log("Error:"@name@"was spawned with a non-matcher as owner!");
        
        Destroy();
    }

    Log("Spawned! Master is:"@Owner.Name);

    foreach AllActors(class'ScriptedPawn', Current)
    {
        c = false;

        Closest.Trace(RL, RN, Current.Location);

        foreach AllActors(class'FriendMonster', FP)
            if ( FP.Closest != None && FP.Closest == Current )
            {
                c = True;

                break;
            }

        if ( c )
            continue;

        if ( Current != None 
        && ( Closest == None || VSize(Current.Location - Owner.Location) < VSize(Closest.Location - Owner.Location) )
        && ( VSize(Vector(Pawn(Owner).ViewRotation) - -RN) < Current.CollisionRadius || RA == Current ) )
            Closest = Current;

        else if ( RA != None )
            log(RA.Name);
    }

    c = false;

    if ( Closest == None )
    {
        Log("Error: No closest monster!");
        Destroy();
        return;
    }
    
    Log(name@"was associated to"@Closest.name);

    a = Closest.Spawn(class'Translocator');

    Closest.GoToState();
    Closest.Team = Pawn(Owner).PlayerReplicationInfo.Team;
    Closest.Hated = None;
    Closest.bHateWhenTriggered = True;
    Closest.Intelligence = BRAINS_HUMAN;
    Closest.OrderObject = Owner;
    Closest.GoToState('Roaming');
    Closest.SpecialGoal = Owner;
    Closest.Aggressiveness = 1;
	Closest.Tag = 'None';
	
    if ( Closest.FindInventoryType(class'Translocator') == None )
    {
        if ( !Closest.AddInventory(Inventory(a)) )
            a.Destroy();

        log("Added translocator. New "@Closest@"stats: Weapon="$Closest.Weapon$",FindInventoryType(class'Translocator')="$Closest.FindInventoryType(class'Translocator'));

        if ( Closest.Weapon == a || Closest.FindInventoryType(class'Translocator') == None )
        {
            Closest.Weapon = None;

            Closest.ChangedWeapon();

            a.Destroy();
        }

        if ( a.IsInState('Pickup') )
            a.Destroy();
    }
    
    if ( Closest.AttitudeTo(Pawn(Owner)) != ATTITUDE_Friendly )
        log("Warning: Attitude from"@Closest.Name@"to"@Owner.Name@"isn't friendly:"@Closest.AttitudeTo(Pawn(Owner)));

    SetTimer(0.1, True);
}

function TranslocDest FindTranslocationPoint(Actor OldPoint)
{
    local TranslocDest TD;
    local TranslocDest Candidates[64];
    local int i;

    foreach AllActors(class'TranslocDest', TD)
        if (
            VSize(TD.Location - Closest.Location) < VSize(OldPoint.Location - Closest.Location)
            && !Closest.ActorReachable(OldPoint)
            && TD != None
        )
        {
            log("Found translocation target candidate:"@TD);
            Candidates[i] = TD;

            i += 1;
        }

    if ( i == 0 )
        return None;

    return Candidates[Rand(i - 1)];
}

function bool TargetEnemy(Pawn R[32], int Size)
{
    local Pawn T;

    if ( R[0] != None )
    {
        while ( T == None )
            T = R[Rand(Size - 1)];
            
        if ( OldTarget != T )
            Log("Targetting"@Closest@"to:"@T);

        OldTarget = T;
    
        Closest.SetEnemy(T);
        Closest.Trigger(self, T);
		
        Closest.GoToState('Attacking');
    }

    else return True;

    return False;
}

function bool SetPawnState(name NextState, string StateDescription, optional name Label, optional bool bAlreadyInStateCheck)
{
    local int i;

    for ( i = 0; i < 32; i++ )
    {
        if ( UninterruptableStates[i] == 'None' )
            break;

        if ( Closest.IsInState(UninterruptableStates[i]) )
            return False;                   
    }

    if ( !Closest.IsInState(NextState) || !bAlreadyInStateCheck )
    {
        log(Closest.Name@"is now in the"@StateDescription@"(technically"@NextState$") state! Previous state was"@Closest.GetStateName()$".");

        if ( Label != 'None' )
            Closest.GoToState(NextState, Label);

        else
            Closest.GoToState(NextState);

        return True;
    }

    else return False;
}

simulated event bool CheckSeekable(Pawn Other, Pawn FireAgent)
{
	if ( FireAgent == None )
		return True;

	return (
		Other != None
		&& Other != Closest
		&& FastTrace(Other.Location)
		&& Other != FireAgent
		&& !Other.IsInState('Dying')
		&&
		(
			(
				ScriptedPawn(Other) == None
				|| (
					ScriptedPawn(Other).AttitudeTo(FireAgent) != ATTITUDE_Friendly
					&& ScriptedPawn(Other).AttitudeTo(FireAgent) != ATTITUDE_Ignore
					&& ScriptedPawn(Other).AttitudeTo(FireAgent) != ATTITUDE_Follow
					&& ScriptedPawn(Other).AttitudeTo(FireAgent) != ATTITUDE_Fear
				)
			)
			&&
			(
				!Level.Game.bTeamGame
				|| Other.PlayerReplicationInfo == None
				|| FireAgent.PlayerReplicationInfo == None
				|| Other.PlayerReplicationInfo.Team != FireAgent.PlayerReplicationInfo.Team
			)
		)
	);
}

function Timer()
{
    local Pawn P;
    local Pawn R[32];
    local int  i;
    local Pawn T;
    local FriendMonster F;
    local bool c;
    local bool e;
    local FearSpot FS;
    local bool bMasterInDanger;
    local bool bActorsCollide;
    
    if ( Closest.AttitudeToPlayer != ATTITUDE_Friendly )
        Closest.AttitudeToPlayer = ATTITUDE_Friendly;

    if ( (Closest.Enemy != None && Closest.Enemy != Owner && !(Closest.Enemy.IsInState('Dying'))) || Closest.IsInState('VictoryDance') )
        return;
        
    else if ( OldTarget != None && !(OldTarget.IsInState('Dying')) && Closest.Enemy == None )
    {
        Closest.SetEnemy(OldTarget);
        Closest.Enemy = OldTarget;
        
        return;
    }
        
    if ( Owner == None )
    {
        Warn(self$"'s master was destroyed! Destroying self as well.");

        Destroy();
        return;
    }

    if ( Closest.bCanBePissed && Care > 12 )
        Closest.bCanBePissed = False;
        
    if ( Closest.Enemy == None || Closest.Enemy == Owner )
    {
        c = False;

        for ( P = Level.PawnList; P != None; P = P.NextPawn )
        {
            foreach P.AllActors(class'FriendMonster', F)
                if ( F.Closest == P && F.Owner == Owner )
                    continue;
					
			if ( bAggressive && P != Owner && P != Closest && ( CheckSeekable(P, Closest) || CheckSeekable(P, Pawn(Owner)) ) )
				R[i] = P;
    
            else if ( P != None
                && ( ( P.PlayerReplicationInfo != None && P.PlayerReplicationInfo.Team != Pawn(Owner).PlayerReplicationInfo.Team ) || ( !P.bIsPlayer && ( P.Enemy == Owner || P.Instigator == Owner ) ) )
                && P != Owner
                && !P.IsInState('Dying')
                && P != Closest
                && ( Closest.FastTrace(P.Location) || VSize(P.Location - Closest.Location) < P.SightRadius * 2 / P.HearingThreshold )
            )
            {
                R[i] = P;
                i++;
        
                if ( i > 31 )
                {
                    Warn("Not enough space for target:"@P);

                    break;
                }
            }
        }
    
        c = TargetEnemy(R, i);
		
		if ( IsInState('Follow') )
			GoToState('NormalControl');
    }

    else c = True;

    if ( c == oc == False )
        log(Closest@"not following.");

    else if ( c == False && oc == True )
        log(Closest@"finished following.");

    oc = c;
	
    if ( c && ( VSize(Closest.Location - Owner.Location) > 256 || !Closest.FastTrace(Owner.Location) ) )
    {
        SetPawnState(FollowState, "follow");
    
        if ( Closest.MoveTarget == None )
        {
            if ( Closest.MoveTarget == None )
                log(Closest.Name@"is now following"@Owner.Name$"; now finding pathes.");
            
            else if ( FollowTarget == None )
                log(Closest.Name@"is now following"@Owner.Name@"towards"@Closest.MoveTarget.Name);
    
            else if ( Closest.MoveTarget != Owner )
                log(Closest.Name@"is now following"@Owner.Name@"towards"@Closest.MoveTarget.Name$", after"@FollowTarget.Name);

            else
                log(Closest.Name@"is now following"@Owner.Name@"directly!");
 
            if ( Closest.MoveTarget != None )
                FollowTarget = Closest.MoveTarget;

            Closest.MoveTarget = Closest.FindPathTo(Owner.Location);

            if ( Closest.MoveTarget == None )
            {
                Closest.MoveTarget = Owner;
                log(Closest.Name@"is now following"@Owner@"directly.");
            }

            if ( FollowTarget == None )
                log(Closest.Name@"set path component:"@Closest.MoveTarget);
        }
    }
}

function Tick(float TimeDelta)
{
    Closest.Disable('SeePlayer');

    if ( Closest == None || Closest.Name == 'None' || Closest.IsInState('Dying') )
    {
        log(name$"'s monster was destroyed! Destroying self as well.");

        Destroy();
        return;
    }
		
	if ( Closest.Enemy == Owner )
        Closest.Enemy = OldTarget;

    if ( Closest.OldEnemy == Owner )
        Closest.OldEnemy = OldTarget;

    if ( Closest.Hated == Owner )
        Closest.Hated = OldTarget;

    if ( Closest.Instigator == Owner )
        Closest.Instigator = OldTarget;
}

function TriggerAllOfKind(class Kind, optional Pawn TriggerInstigator)
{
    local   Actor   A;

    if ( TriggerInstigator == None )
        TriggerInstigator = Pawn(Owner);

    foreach AllActors(class'Actor', A)
        if ( A.class.name == Kind.name )
            A.Trigger(self, TriggerInstigator);
}

auto state NormalControl
{
} // default "does nothing unusual" state--for now!

/*
state Follow
{
    Begin:
        if ( !bFollow || !( VSize(Closest.Location - Owner.Location) > 256 || !Closest.FastTrace(Owner.Location) ) )
            GoToState('NormalControl');

        log(Name@"entered the Follow state; regulating the follow for"@Owner@"by"@Closest);

        Closest.Skill *= FollowSkill;
        SetPawnState(FollowState, "follow", FollowLabel);

        while ( ( VSize(Owner.Location - Closest.Location ) < CoverDistance + Closest.CollisionRadius ) && bFollow && Closest.IsInState(FollowState) && !Closest.IsInState(FollowedState) )
        {
            NextRunTarget = Closest.FindPathTo(Owner.Location);

            if ( NextRunTarget == None )
                NextRunTarget = Owner;

            TD = FindTranslocationPoint(NextRunTarget);

            if ( Closest.FindInventoryType(class'Translocator') != None && TD != None )
            {
                OldWeap = Closest.Weapon;

                Closest.Weapon = Weapon(Closest.FindInventoryType(class'Translocator'));

                Closest.ViewRotation = Rotator(TD.Location - Closest.Location);
                Closest.FireWeapon();

                while ( VSize(Translocator(Closest.Weapon).TTarget.Location - TD.Location) > TD.CollisionRadius )
                    Sleep(0.1);

                Closest.Weapon.AltFire(1.0);

                NextRunTarget = TD;

                Closest.Weapon = OldWeap;
            }

            SetPawnState(FollowState, "follow continuation", FollowLabel, true);

            Closest.TurnToward(NextRunTarget);
            Closest.MoveToward(NextRunTarget, FollowSpeed);
            Closest.TurnToward(NextRunTarget);

            Sleep(0.2);
        }

        Closest.Skill /= FollowSkill;
        SetPawnState(FollowedState, "post-follow", FollowedLabel);
        GoToState('NormalControl');
}
*/

defaultproperties
{
     FollowState=Roaming
     FollowLabel=Moving
     FollowedState=Waiting
     InitialPawnState=Roaming
     UninterruptableStates(0)=Teleporting
     UninterruptableStates(1)=Following
     UninterruptableStates(2)=wandering
     UninterruptableStates(3)=Roaming
     FollowSkill=50.000000
     FollowSpeed=1.600000
     PullRunSpeed=3.100000
     CoverDistance=128.000000
     Care=15.000000
     bAggressive=True
     bFollow=True
     bHidden=True
}
