//=============================================================================
// GSaddlePawn.
//
// Makes it possible to ride a Pawn externally.
//=============================================================================
class GSaddlePawn expands Keypoint;

// Detection of ridden
var()	name		RiddenTag;

// Detection of rider
var()	class<Pawn>	RiderTypes[16];
var()	float		MaxRideMassFactor;
var()	float		MinRideHeight;
var()	float		MaxRideHeight;
var()	float		RideDistanceFactor;

// Riding data
var()	float		ActualRideHeight;
var()	float		NormalRidePosition;
var		float		RideMass;
var		Pawn		Riding;
var		Pawn		Rider;
var		AttachInfo	Saddle;

simulated function PostBeginPlay()
{
	foreach AllActors(class'Pawn', Riding, RiddenTag)
	{
		Log(self@"is determining as ridable:"@Riding);

		break;
	}

	if ( Riding == None )
	{
		Destroy();

		return;
	}
	
	SetTimer(0.1, true);
}

simulated function final vector Vec2D(vector V)
{
	V.Z = 0;
	
	return V;
}

simulated function vector UpdateRideLoc()
{
	local Vector RideV;
	local Rotator Up, Forward;

	Forward = Riding.Rotation;
	Forward.Pitch = 0;
	
	Up = Riding.ViewRotation;
	Up.Pitch += 16384;

	RideV += Vector(Up) * ActualRideHeight;
	RideV += Vector(Forward) * NormalRidePosition;

	return RideV;
}

simulated event Timer()
{
	local Pawn Candidate;
	local Pawn CanRide[32];
	local GSaddlePawn S;
	local int i, n, j;
	local bool b;

	if ( Riding == None || Riding.IsInState('Dying') || Riding.IsInState('Gibbed') )
	{
		Destroy();

		return;
	}

	if ( Rider != None && !Rider.IsInState('Dying') && !Rider.IsInState('Gibbed') )
	{
		Saddle.AttachOffset = UpdateRideLoc();

		// if ( Riding.IsInState('Waiting') || Riding.IsInState('StartUp') )
		//     Riding.GoToState('Wandering');

		if ( ScriptedPawn(Riding) != None && Riding.Enemy == Rider )
		{
			if ( Rider.Enemy != None ) // Shared goals.
				Riding.Enemy = Rider.Enemy;

			else if ( Riding.Enemy == Rider )
				Riding.Enemy = None;

			if ( PlayerPawn(Rider) != None ) // Can I have friend?
				ScriptedPawn(Riding).AttitudeToPlayer = ATTITUDE_Friendly;
		}

		if ( ScriptedPawn(Riding) != None && Riding.Enemy == None && ( Riding.Physics == PHYS_Flying ) && !( Riding.IsInState('Roaming') || Riding.IsInState('Wandering') ) )
        {
            if ( Riding.Physics == PHYS_Flying ) // leftover stupidity
                Riding.SetRotation(Rider.ViewRotation);
                
            else
                Riding.SetRotation(Rider.Rotation);
        }

		return;
	}

	if ( Saddle != None )
	{
		if ( Rider != None )
		{
			Rider.SetCollision(true, true, true);

			Rider = None;
		}

		Riding.Mass -= RideMass;

		Saddle.Destroy();
		Saddle = None;
	}

	for ( Candidate = Level.PawnList; Candidate != None; Candidate = Candidate.NextPawn )
	{
		b = false;
	
		if ( Candidate == Riding )
			continue;
	
		for ( j = 0; j < 16; j++ )
		{
			if ( RiderTypes[j] == None )
				break;
		
			if ( Candidate.IsA(RiderTypes[j].name) )
				b = true;		
		}
				
		if ( !b )
		{
			Log(Candidate.class.name);
			continue;
		}
	
		if (
			MinRideHeight < Candidate.Location.Z - Riding.Location.Z
			&& Candidate.Location.Z - Riding.Location.Z < MaxRideHeight
			&& (!Candidate.IsInState('Dying'))
			&&
			(
				MaxRideMassFactor <= 0
				|| Candidate.Mass <= MaxRideMassFactor * Riding.Mass
			)
			&&
			(
				RideDistanceFactor <= 0
				|| VSize(Vec2D(Candidate.Location - Riding.Location)) <= (Riding.CollisionRadius + Candidate.CollisionRadius) * RideDistanceFactor
			)
		)
		{
			// Log("New candidate for riding"@Riding$":"@Candidate);

			if ( i > 31 )
            {
				Log("Warning: Max candidates overflow with:"@Candidate);
            }

			else
			{
				CanRide[i] = Candidate;
				i++;
			}
		}
		
		else
		{
			/*
			Log(
				Candidate$":"
				@ MinRideHeight < Candidate.Location.Z - Riding.Location.Z
				@ Candidate.Location.Z - Riding.Location.Z < MaxRideHeight
				@ (
					MaxRideMassFactor <= 0
					|| Candidate.Mass <= MaxRideMassFactor * Riding.Mass
				)
				@ (
					RideDistanceFactor <= 0
					|| VSize(Vec2D(Candidate.Location - Riding.Location)) <= (Riding.CollisionRadius + Candidate.CollisionRadius) * RideDistanceFactor
				)
				@ !Candidate.IsInState('Dying')
			);
			*/
		}
	}
	
	if ( i > 31 )
	{
		Warn("Found"@i-31@"overflowing candidates: ignoring (max 32).");
	}

	for ( n = 0; n < i; n++ )
		if ( Rider == None || VSize(CanRide[n].Location - Riding.Location) < VSize(Rider.Location - Riding.Location) )
			Rider = CanRide[n];

	if ( Rider != None )
	{
		Log(self@"made"@Riding@"accept as rider:"@Rider);

		Saddle = class'AttachInfo'.static.CreateAttachment(Riding, Rider);
		Rider.SetCollision(true, false, false);
		Riding.Mass += Rider.Mass;
		RideMass = Rider.Mass;

		Saddle.AttachOffset = UpdateRideLoc();
	}
}

defaultproperties
{
     RiderTypes(0)=Class'Engine.Pawn'
     MaxRideMassFactor=1.200000
     MinRideHeight=-1.000000
     MaxRideHeight=2.000000
     RideDistanceFactor=1.800000
     ActualRideHeight=0.600000
     NormalRidePosition=-0.100000
     bStatic=False
}
