//=============================================================================
// GusLightmapInfo.
//=============================================================================
class GusLightmapInfo expands Info;

var()	float			LightmapSize;
var()	float			LightmapPrecision;
var()	float			AttenuationFactor;
var()	float			LightFactor;

var		GusLightmapChunk	Chunks;

function StartedTracing(Light Other, Vector Normal);

function UpdateLightmaps()
{
	local GusLightmapChunk CurrentChunk;
	local GusLightmapChunk PrevTrace, CurrTrace, CleanupChunk;
	local Light CurrentLight;
	local int CleanedUpChunks;
	local vector StartV, OldV;

	local vector Loc, Nrm;

	foreach AllActors(class'GusLightmapChunk', CurrentChunk)
		CurrentChunk.Destroy();

	foreach AllActors( class'Light', CurrentLight )
	{
		CurrentLight.LightType = LT_None;

		StartV = CurrentLight.Location;

		// Log("Tracing lightmap for"@CurrentLight);

		for ( OldV.X = -1; OldV.X <= 1; OldV.X += LightmapSize )
			for ( OldV.Y = -1; OldV.Y <= 1; OldV.Y += LightmapSize )
				for ( OldV.Z = -1; OldV.Z <= 1; OldV.Z += LightmapSize )
				{
					if ( OldV == vect(0.0, 0.0, 0.0) )
						continue;

					// Log("| Tracing with normal:"@OldV);

					CurrentChunk = Spawn(
						class'GusLightmapChunk',
						self,,
						StartV + (OldV * LightmapPrecision)
					);

					CurrTrace = CurrentChunk;

					if ( PrevTrace != None )
						PrevTrace.NextTrace = CurrentChunk;

					CurrentChunk.Precision = LightmapPrecision;
					CurrentChunk.Root = CurrentChunk;
					CurrentChunk.MyNormal = OldV;
					CurrentChunk.LightIntensity = CurrentLight.LightBrightness * LightFactor;
					CurrentChunk.Attenuation = AttenuationFactor * CurrentLight.LightRadius / 64;

					CurrentChunk.SetUp();

					for (
						StartedTracing(CurrentLight, OldV);
						Trace(Loc, Nrm, CurrentChunk.Location, CurrentChunk.Location + (OldV * LightmapPrecision)) == None;
						CurrentChunk = CurrentChunk.Add()
					)
					{
						CurrentChunk.Precision = LightmapPrecision;
						CurrentChunk.MyNormal = OldV;
						CurrentChunk.LightIntensity = CurrentLight.LightBrightness * LightFactor;
						CurrentChunk.Attenuation = 1.0 / CurrentLight.LightRadius;
					}

					// Log("> Traced"@CurrTrace.TraceSize()@"chunks at normal"@OldV);

					PrevTrace = CurrTrace;
				}
	}
}

function float GetBrightnessAt(vector Loc)
{
	local GusLightmapChunk	Current;
	local float				Dist;
	local float 			f;

	for ( Current = Chunks; Current != None; Current = Chunks.Next )	
		if ( Current.FastTrace(Loc) )
			f += Current.LightIntensity / ((VSize(Current.Location - Loc) / LightmapPrecision) * Current.Attenuation);

	return f;
}

function PreBeginPlay()
{
	local GusLightmapLogMarker GLLM;

	UpdateLightmaps();

	foreach AllActors(class'GusLightmapLogMarker', GLLM)
		Log("The brightness at"@GLLM.Location@"is:"@GetBrightnessAt(GLLM.Location));
}

defaultproperties
{
     LightmapSize=0.750000
     LightmapPrecision=64.000000
     AttenuationFactor=0.000100
     LightFactor=0.450000
}
