#exec mesh import mesh=MBeaconWeap anivfile=Models\MBeaconWeap_a.3d datafile=Models\MBeaconWeap_d.3d x=0 y=0 z=0 mlod=1

#exec mesh sequence mesh=MBeaconWeap seq=All startframe=0 numframes=37 rate=19
#exec mesh sequence mesh=MBeaconWeap seq=Still startframe=0 numframes=1 rate=1
#exec mesh sequence mesh=MBeaconWeap seq=Fire startframe=0 numframes=11 rate=18
#exec mesh sequence mesh=MBeaconWeap seq=Down startframe=11 numframes=14 rate=28
#exec mesh sequence mesh=MBeaconWeap seq=Select startframe=25 numframes=12 rate=21

#exec meshmap new meshmap=MBeaconWeap mesh=MBeaconWeap
#exec meshmap scale meshmap=MBeaconWeap x=0.1 y=0.1 z=0.2

#exec texture import name=JMBeaconWeap file=Textures\JMBeaconWeap.pcx group=Skins
#exec meshmap settexture meshmap=MBeaconWeap num=0 texture=JMBeaconWeap

class MushBeacon extends TournamentWeapon;


var bool bRating;

simulated function Projectile ProjectileFire(class<projectile> ProjClass, float ProjSpeed, bool bWarn)
{
	local Projectile pj;
	
	FireOffset.X += 15;
	
	while ( pj == None )
	{
		FireOffset.X -= 5;
	
		if ( FireOffset.X < -20 )
			return None;
		
		pj = Super.ProjectileFire(ProjClass, ProjSpeed, bWarn);
	}
	
	FireOffset.X = Default.FireOffset.X;
	
	pj.Instigator = Pawn(Owner);
	
	return pj;
}

simulated function Tick(float TimeDelta)
{
	local Pawn p;
	
	Super.Tick(TimeDelta);
		
	if ( Owner == None || Owner.IsInState('Dying') || ( MushMatch(Level.Game) == none && Pawn(Owner).PlayerReplicationInfo != none && Pawn(Owner).PlayerReplicationInfo.Deaths > 0 ) )
	{
		Destroy();
		return;
	}
}

simulated function float PassiveRating()
{
	local Pawn P;
	local float Score;
	
	Score = -50;
	
	if ( bRating )
		return 0;
		
	bRating = true;

	if ( ( Pawn(Owner).Enemy != None && Pawn(Owner).Enemy.FindInventoryType(class'GusPackII.SpottedMush') != None ) || Pawn(Owner) == None || MushMatch(Level.Game) == None )
		return -50000;
	
	if ( MushMatch(Level.Game).bHasHate )
		for ( P = Level.PawnList; P != None; P = P.NextPawn )
			if ( MushMatch(Level.Game).bHasHate && class'HateList'.static.CheckHate(P, Pawn(Owner)) && P.FindInventoryType(class'GusPackII.NotMush') == None && ( !(MushMatch(Level.Game).bHasBeacon) || class'BeaconList'.static.CheckBeacon(P) == None ) ) 
				Score += Max(200, 1024 - VSize(Owner.Location - P.Location)) / 4;
	
	bRating = false;
	
	return Score;
}

simulated function float RateSelf(out int bUseAltMode)
{
	local Pawn P;
	local float Score, BestScore, ThisScore;
	local Pawn BestPawn;
	local int unused;
	
	Score = -50;
	bUseAltMode = 0;
	
	if ( bRating )
		return 0;
		
	bRating = true;
	
	if ( Pawn(Owner) == None || MushMatch(Level.Game) == None )
		return -50000;
	
	if ( MushMatch(Level.Game).bHasHate )
		for ( P = Level.PawnList; P != None; P = P.NextPawn )
			if ( MushMatch(Level.Game).bHasHate && class'HateList'.static.CheckHate(P, Pawn(Owner)) && ( !(MushMatch(Level.Game).bHasBeacon) || class'BeaconList'.static.CheckBeacon(P) == None ) && P.FindInventoryType(class'SpottedMush') == None )
			{	
				ThisScore = Max(200, 1024 - VSize(Owner.Location - P.Location)) / 4;
					
				if ( ThisScore > BestScore )
				{
					BestScore = ThisScore;
					BestPawn = P;
				}
				
				Score += ThisScore;
			}

	if ( BestPawn != None )
		Bot(Owner).SetEnemy(BestPawn);
		
	bRating = false;
	
	return Score;
}

simulated function PlayFiring()
{
	PlayAnim('Fire', 1.4, 0.1);
	PlayOwnedSound(FireSound, SLOT_Misc, 1.0);
}

simulated function PlayIdle()
{
	PlayAnim('Still', 0.35, 0.5);
}

defaultproperties
{
     AmmoName=Class'GusPackII.MushBeaconAmmo'
     PickupAmmoCount=15
     FireOffset=(X=32.000000)
     ProjectileClass=Class'GusPackII.MushBeaconProj'
     AltProjectileClass=Class'GusPackII.MushBeaconProj'
     FireSound=Sound'Botpack.BioRifle.GelHit'
     SelectSound=Sound'Botpack.enforcer.Cocking'
     DeathMessage="%k got crazy and killed %o with some Mush Beacon Gun..."
     PickupMessage="You got the Mush Beacon Gun."
     ItemName="Mush Beacon"
     PlayerViewOffset=(X=10.100000,Y=-6.000000,Z=-5.500000)
     PlayerViewMesh=LodMesh'GusPackII.MBeaconWeap'
     PickupViewMesh=LodMesh'GusPackII.MBeaconWeap'
     ThirdPersonMesh=LodMesh'GusPackII.MBeaconWeap'
     ThirdPersonScale=0.6
     PlayerViewScale=0.3
     StatusIcon=Texture'Botpack.Icons.UsePulse'
     MaxDesireability=0.000000
     PickupSound=Sound'Botpack.enforcer.Cocking'
     Icon=Texture'Botpack.Icons.IconPulse'
     Mesh=LodMesh'GusPackII.MBeaconWeap'
     Mass=65.000000
}
