/*
 * If I can't do it in ZDoom,
 * let's do it in UT99 :D
 */

#exec AUDIO IMPORT FILE="Sounds\mm_found.wav" NAME="FoundMush" GROUP="MushMatch"
#exec AUDIO IMPORT FILE="Sounds\mm_suspect.wav" NAME="Suspected" GROUP="MushMatch"

class MushMatch expands TeamGamePlus;

var string DiscoveredMusic;
var bool bSelected, bHumanVictory, bMatchEnd, bHasHate, bHasBeacon;
var string RTeamNames[2];
var byte winTeam;

function HateList RegisterHate(Pawn Victim, Pawn InstigatedBy)
{
    local HateList h;
    
    bHasHate = true;
    
    if ( class'HateList'.static.CheckHate(InstigatedBy, Victim) )
        return None;
    
    h = InstigatedBy.Spawn(class'HateList');
    h.Hater = Victim;

    InstigatedBy.AddInventory(h);
    
    return h;
}

simulated function PreBeginPlay()
{
    local navigationPoint np;

    Super.PreBeginPlay();
    
    foreach AllActors(class'navigationPoint', np)
    {
        if ( FRand() < 0.05 ) Spawn(class'SporeCanister',,, np.location);
        if ( FRand() < 0.05 ) Spawn(class'MushBeaconAmmo',,, np.location);
    }
}

simulated function PostBeginPlay()
{
    local Pawn P;

    Super.PostBeginPlay();
    
    SetTimer(0.5, true);
    
    for ( P = Level.PawnList; P != none; P = P.nextPawn )
        if ( P.bIsPlayer && p.PlayerReplicationInfo != none )
            p.PlayerReplicationInfo.Team = 0;
}


function ScoreKill(Pawn Killer, Pawn Other)
{
    local vector hLoc, hMom;
    local int dmg;
    
    local PlayerMonitor p;
    
    hLoc = Other.Location;
    hMom = vect(0,0,0);
    dmg = 32767;
    
    if ( !bSelected )
    {
        Other.PlayerReplicationInfo.Deaths -= 1;
    
        DiscardInventory(Other);
        return;
    }
        
    foreach Other.ChildActors(class'GusPackII.PlayerMonitor', p) p.Instigator = Killer;
        
    if ( other.PlayerReplicationInfo == None || Killer.PlayerReplicationInfo == None )
    {
        DiscardInventory(Other);
        return;
    }
    
    if ( !Killer.bIsPlayer || !Other.bIsPlayer )
    {
        Super(DeathmatchPlus).ScoreKill(Killer, Other);
        DiscardInventory(Other);
        
        return;
    }
    
    MushMatchMutator(BaseMutator).MutatorScoreKill(Killer, Other);
    
    if ( Killer.PlayerReplicationInfo.Team != Other.PlayerReplicationInfo.Team )
        Killer.PlayerReplicationInfo.Score += 1;
        
    else
        Killer.PlayerReplicationInfo.Score -= 1;

    if ( bMatchEnd )
        return;

    if ( PlayerPawn(Other) != none )
        PlayerPawn(Other).PlayerReplicationInfo.Deaths += 1;
        
    DiscardInventory(Other);
    CheckEnd();
}
    
function CheckEnd()
{
    local bool h, m;
    local Pawn p;
    
    if ( !bSelected || bMatchEnd )
        return;
    
    for ( p = Level.PawnList; p != none; p = p.nextPawn )
        if ( p.bIsPlayer && p.PlayerReplicationInfo != none && !p.IsInState('Dying') && p.PlayerReplicationInfo.Deaths == 0 )
        {
            if ( p.PlayerReplicationInfo.Team == 0 )
                h = true;
        
            if ( p.PlayerReplicationInfo.Team == 1 )
                m = true;
        }
        
    if ( !h || !m )
    {
        if ( h )
            winTeam = 0;
            
        else
            winTeam = 1;
    
        Broadcastmessage("The"@RTeamNames[winTeam]@GameEndedMessage);
        SetEndCams("teamstand");
    }
}

function bool SetEndCams(string Reason)
{
    local float EndTime;
    local Pawn P, RWinner;
    local PlayerPawn Player;

    bMatchEnd = true;
    EndTime = Level.TimeSeconds + 3.0;
    GameReplicationInfo.GameEndedComments = "The"@RTeamNames[winTeam]@GameEndedMessage;
    log("Game ended at "$EndTime);
    
    for ( P=Level.PawnList; P!=None; P=P.nextPawn )
    {
        if ( P.bIsPlayer && P.PlayerReplicationInfo != none && P.PlayerReplicationInfo.Team == winTeam && P.PlayerReplicationInfo.Deaths <= 0 && ( RWinner == none || P.PlayerReplicationInfo.Score > RWinner.PlayerReplicationInfo.Score ) )
            RWinner = P;
    }
    
    for ( P=Level.PawnList; P!=None; P=P.nextPawn )
    {
        Player = PlayerPawn(P);
        
        if ( Player != None )
        {
            if (!bTutorialGame)
                PlayWinMessage(Player, (Player.PlayerReplicationInfo.Team == winTeam));
                
            Player.bBehindView = true;
            
            if ( RWinner != None )
                Player.ViewTarget = RWinner;
                
            else
                Player.ViewTarget = Player;
                
            Player.ClientGameEnded();
        }

        P.GotoState('GameEnded');
    }
    
    CalcEndStats();
    GotoState('GameEnded');
    
    return true;
}

function BeaconList StrapBeacon(Pawn Other, optional Pawn Suspector)
{
    local BeaconList b;

    if ( Other == None )
        return None;
        
    bHasBeacon = true;
    
    if ( class'BeaconList'.static.CheckBeacon(Other) != None )
        return class'BeaconList'.static.CheckBeacon(Other);
        
    BroadcastMessage(Suspector.PlayerReplicationInfo.PlayerName@"attached a suspicion beacon on"@Other.PlayerReplicationInfo.PlayerName$"!", true, 'CriticalEvent');
    
    if ( PlayerPawn(Other) != None )
        Other.PlayOwnedSound(sound'GusPackII.MushMatch.Suspected');
    
    b = Other.Spawn(class'BeaconList');
    b.Instigator = Suspector;

    Other.AddInventory(b);
    
    return b;
}

event PostLogin( playerpawn NewPlayer )
{
    Super.PostLogin(NewPlayer);

    if ( Level.NetMode != NM_Standalone )
    {
        if ( FRand() < 0.25 )
            NewPlayer.PlayerReplicationInfo.Team = 1;
        
        else
            NewPlayer.PlayerReplicationInfo.Team = 0;
    }
}

function StartMatch()
{	
    Super.StartMatch();

    GoToState('GameStarted');
}

function byte AssessBotAttitude(Bot aBot, Pawn Other)
{
    local Pawn P;

    if ( Other.PlayerReplicationInfo == none )
        return 2;
        
    if ( aBot.PlayerReplicationInfo.Team == 0 )
    {
        if (
            Other.bIsPlayer
            && Other.FindInventoryType(class'GusPackII.NotMush') == None
            &&
            (
                (
                    Other.PlayerReplicationInfo != none
                    && Other.PlayerReplicationInfo.Team == 1
                    && Other.FindInventoryType(class'GusPackII.SpottedMush') != none
                )
                ||
                (
                    bHasHate
                    && class'HateList'.static.CheckHate(Other, aBot)
                )
                ||
                (
                    bHasBeacon
                    && class'BeaconList'.static.CheckBeacon(Other) != None
                    && class'BeaconList'.static.CheckBeacon(Other).Instigator.FindInventoryType(class'GusPackII.SpottedMush') == None
                )
            )
        )
            return 1;
        
        if ( FRand() < 0.3 )
            return 3;
        
        return 2;
    }
    
    if ( aBot.PlayerReplicationInfo.Team == 1 )
    {
        if ( Other.PlayerReplicationInfo.Team == 0 )
        {
            if ( aBot.FindInventoryType(class'GusPackII.SpottedMush') != None || ( bHasHate && class'HateList'.static.CheckHate(aBot, Other) ) )
                return 1;
        
            if ( Other.CanSee(aBot) )
                return 2;
        
            for ( P = Level.PawnList; P != None; P = P.NextPawn )
                if ( P.bIsPlayer && P.PlayerReplicationInfo != None && P.PlayerReplicationInfo.Team == 0 && P.LineOfSightTo(aBot) && P != Other )
                    return 2;
        
            if ( Other.MoveTarget != None && VSize(Other.Location - Other.MoveTarget.Location) > 512 && VSize(Normal(Other.MoveTarget.Location - Other.Location) - Vector(aBot.ViewRotation)) > 0.55 && aBot.FindInventoryType(class'GusPackII.Sporifier') != None )
                aBot.Weapon = Weapon(aBot.FindInventoryType(class'GusPackII.Sporifier'));
        
            return 1;
        }
            
        return 3;
    }
}

function bool SpotMush(Pawn Other, Pawn Finder)
{
    local SpottedMush s;
    local Music mus;

    if ( !Other.bIsPlayer || Other.PlayerReplicationInfo == none || Other.FindInventoryType(class'GusPackII.Spottedmush') != none )
        return false;

    s = Other.Spawn(class'GusPackII.SpottedMush');
        
    Other.AddInventory(s);
    
    Broadcastmessage(Other.PlayerReplicationInfo.Playername@"was discovered as mush by"@Finder.PlayerReplicationInfo.PlayerName$"!", true, 'CriticalEvent');
    
    if ( PlayerPawn(Other) != None && DiscoveredMusic != "" )
    {
        mus = Music(DynamicLoadObject(DiscoveredMusic, class'Music'));
        
        if ( mus != None )
        {
            Log("Playing discovered music:"@DiscoveredMusic);
            PlayerPawn(Other).ClientSetMusic(mus, 0, 0, MTRAN_FastFade);
            Spawn(class'GusPackII.MushMusic', Other);
        }
        
        else
            Warn(mus@"not found!");
    }
    
    if ( PlayerPawn(Other) != None )
        Other.PlayOwnedSound(sound'GusPackII.MushMatch.FoundMush');
    
    return true;
}

function bool RestartPlayer(Pawn aPlayer)
{
    if ( bMatchEnd )
    {
        if ( PlayerPawn(aPlayer) != None ) EndGame("unknownvictory");
    
        return false;
    }

    if ( aPlayer.PlayerReplicationInfo.Deaths > 0 && bSelected )
        return false;

    aPlayer.PlayerReplicationInfo.Team = 0;

    return Super.RestartPlayer(APlayer);
}

function bool SetUpBot(Bot b)
{
    if ( b == none )
    {
        Warn("Added bot not found!");
        
        return true;
    }
    
    b.PlayerReplicationInfo.Team = 0;
    
    return true;
}

function bool AddBot()
{
    local Pawn p;
    local Bot b;

    if ( !Super.AddBot() )
        return false;
    
    for ( p = Level.PawnList; p != none; p = p.nextPawn )
        if ( Bot(p) != none ) b = Bot(p);
        
    return SetUpBot(b);
}

function bool ForceAddBot()
{
    local Pawn p;
    local Bot b;

    if ( !Super.ForceAddBot() )
        return false;
    
    for ( p = Level.PawnList; p != none; p = p.nextPawn )
        if ( Bot(p) != none ) b = Bot(p);
        
    return SetUpBot(b);
}

function bool ChangeTeam(Pawn Other, int N) 
{
    if ( MushCount() > CountPlayers() / 4 )
        Other.PlayerReplicationInfo.Team = 0;
        
    else
        Other.PlayerReplicationInfo.Team = 1;
        
    return true;
}

function int CountPlayers()
{
    local int i;
    local Pawn p;
    
    for ( p = Level.PawnList; p != none; p = p.NextPawn )
        if ( p.bIsPlayer )
            i++;
            
    return i;
}

function int MushCount()
{
    local int i;
    local Pawn p;
    
    for ( p = Level.PawnList; p != none; p = p.NextPawn )
        if ( p.bIsPlayer && p.PlayerReplicationInfo != none && p.PlayerReplicationInfo.Team == 1 )
            i++;
            
    return i;
}

auto state Idle {}

state GameStarted
{
    function Selected()
    {
        local Pawn p;
        local Sporifier sp;
    
        foreach AllActors(class'Sporifier', sp)
            sp.MushSelected();
    
        for ( p = Level.PawnList; p != none; p = p.NextPawn )
            if ( p.bIsPlayer && p.PlayerReplicationInfo != none )
            {
                // Log("Spawning monitor at:"@p@p.PlayerReplicationInfo.PlayerName);
                Spawn(class'GusPackII.PlayerMonitor', p,, p.Location);
            }
    }

    function SelectTraitor()
    {
        local Pawn p, c;
        local bool b;
        local Sporifier s;
        local Weapon w;
        
        while ( p == none )
        {
            for ( c = Level.PawnList; c != none; c = c.nextPawn )
            {
                if ( c.bIsPlayer )
                    b = true;
            
                if ( FRand() < 0.08 && c.bIsPlayer && c.PlayerReplicationInfo != none && c.PlayerReplicationInfo.Team == 0 )
                    p = c;
            }
            
            if ( !b )
                Error("Match has no players!");
        }
                    
        p.PlayerReplicationInfo.Team = 1;
        w = p.Weapon;
        DeathMatchPlus(Level.Game).GiveWeapon(p, "GusPackII.Sporifier");
        p.Weapon = w;
        
        // Log(p@"("$p.PlayerReplicationInfo.PlayerName$") is now mush!");
    }
    
Begin:
    Sleep(10.0);

    while ( CountPlayers() > MushCount() * 4 || MushCount() == 0 )
        SelectTraitor();
    
    Selected();
    Broadcastmessage("Selected this match's mushes!", true, 'CriticalEvent');
    bSelected = true;
}

state GameEnded
{
Begin:
    Sleep(8);

    if ( Level.NetMode != NM_Standalone )
        EndGame("net end game");
}

defaultproperties
{
     RTeamNames(0)="Humans"
     RTeamNames(1)="Mushes"
     FriendlyFireScale=1.000000
     MaxAllowedTeams=2
     TeamColor(0)="Unknown"
     TeamColor(1)="Unknown"
     StartUpMessage="Kill any suspiciously mush players to win the match! Don't trust team colors."
     GameEndedMessage="win the match!"
     MaxCommanders=0
     bCoopWeaponMode=True
     ScoreBoardType=Class'GusPackII.MushMatchScoreBoard'
     HUDType=Class'GusPackII.MushMatchHUD'
     BeaconName="MUSH"
     GameName="Mush Match"
     MutatorClass=Class'GusPackII.MushMatchMutator'
     DiscoveredMusic="GusPackII_Discovered.GusPackII_Discovered"
}
