class MushMatchMutator extends DMMutator;

var bool bBeginplayed;

simulated function string TeamText(PlayerReplicationInfo PRI, PlayerPawn Other)
{
    if ( Pawn(PRI.Owner).IsInState('Dying') || PRI.Deaths > 0 )
    {
        if ( PRI.Team == 1 )
            return "Mush (dead)";
            
        if ( PRI.Team == 0 )
            return "Human (dead)";
    }

    if ( MushMatch(Level.Game).bSelected && ( Other.PlayerReplicationInfo == PRI || Other.PlayerReplicationInfo.Team == 1 || MushMatch(Level.Game).bMatchEnd || Other.PlayerReplicationInfo.Deaths > 0 ) )
    {
        if ( PRI.Team == 1 )
        {
            if ( MushMatch(Level.Game).bHasBeacon && class'BeaconList'.static.CheckBeacon(Pawn(PRI.Owner)) != None )
                return "Mush (susp.)";
            
            if ( Pawn(PRI.Owner).FindInventoryType(class'GusPackII.SpottedMush') == none )
                return "Mush (unk.)";
        }
            
        if ( PRI.Team == 0 )
        {	
            if ( MushMatch(Level.Game).bHasBeacon && class'BeaconList'.static.CheckBeacon(Pawn(PRI.Owner)) != None )
                return "Human (susp.)";
                
            if ( Pawn(PRI.Owner).FindInventoryType(class'GusPackII.NotMush') != none )
                return "Human (cert)";
            
            if ( Pawn(PRI.Owner).FindInventoryType(class'GusPackII.NotMush') == None )
                return "Human";
        }
    }
    
    if ( MushMatch(Level.Game).bHasBeacon && class'BeaconList'.static.CheckBeacon(Pawn(PRI.Owner)) != None )
        return "Suspected";
	
	if ( PRI.Team == 0 && MushMatch(Level.Game).bSelected && Pawn(PRI.Owner).FindInventoryType(class'GusPackII.NotMush') != none )
		return "Human (cert)";

	if ( PRI.Team == 1 && MushMatch(Level.Game).bSelected && Pawn(PRI.Owner).FindInventoryType(class'GusPackII.SpottedMush') != none )
		return "Mush!";
	
	if ( MushMatch(Level.Game).bHasBeacon && class'BeaconList'.static.CheckBeacon(Pawn(PRI.Owner)) != None && class'BeaconList'.static.CheckBeacon(Pawn(PRI.Owner)).Instigator.FindInventoryType(class'GusPackII.SpottedMush') == None )
		return "Suspected";
	
	return "Unknown";
}

function PostBeginPlay()
{
	if ( bBeginplayed )
		return;
	
	Super.PostBeginPlay();
	
	Level.Game.RegisterDamageMutator(self);
	
	bBeginplayed = true;
}

function bool HandleEndGame()
{
	return true;
}

simulated function MutatorTakeDamage(out int ActualDamage, Pawn Victim, Pawn InstigatedBy, out Vector HitLocation, out Vector Momentum, name DamageType)
{
	if ( InstigatedBy == None || Victim == None || InstigatedBy == Victim )
		return;
		
    if ( Victim.CanSee(InstigatedBy) )
        CheckSuspects(InstigatedBy, Victim);
		
	if ( NextDamageMutator != None )
		NextDamageMutator.MutatorTakeDamage(ActualDamage, Victim, InstigatedBy, HitLocation, Momentum, DamageType);
}

simulated function ModifyPlayer(Pawn Other)
{
	local Weapon w;

	w = Other.Weapon;
	DeathMatchPlus(Level.Game).GiveWeapon(Other, "GusPackII.MushBeacon");
	if ( w != None ) Other.Weapon = w;
	
	if ( NextMutator != None )
		NextMutator.ModifyPlayer(Other);
}

simulated function CheckSuspects(Pawn InstigatedBy, Pawn Victim)
{
	local Pawn P;

	for ( P = Level.PawnList; P != None; P = P.NextPawn )
		if ( P != None && P.bIsPlayer && P.PlayerReplicationInfo != None && (!P.IsInState('Dying')) && P != Victim && P != InstigatedBy && P.PlayerReplicationInfo.Deaths <= 0 && P.CanSee(InstigatedBy) && P.LineOfSightTo(Victim) && P.PlayerReplicationInfo.Team == 0 && ( !(MushMatch(Level.Game).bHasHate) || !(class'HateList'.static.CheckHate(InstigatedBy, P)) ) )
		{
			// Log(p.PlayerReplicationInfo.PlayerName@"suspects"@InstigatedBy.PlayerReplicationInfo.PlayerName);
		
			MushMatch(Level.Game).RegisterHate(P, InstigatedBy);
		}

	if ( Victim.LineOfSightTo(InstigatedBy) && Victim.PlayerReplicationInfo.Team == 0 && ( !(MushMatch(Level.Game).bHasHate) || !(class'HateList'.static.CheckHate(InstigatedBy, Victim)) ) )
	{
		// Log(Victim.PlayerReplicationInfo.PlayerName@"suspects"@InstigatedBy.PlayerReplicationInfo.PlayerName);
	
		MushMatch(Level.Game).RegisterHate(Victim, InstigatedBy);
	}
}

simulated function TellTeam(byte PTeam, string PName, optional bool bAutopsy)
{
	if ( bAutopsy )
	{
		if ( PTeam == 0 )
			Broadcastmessage(PName@"was found dead! Autopsy revealed he/she was a human!", true, 'CriticalEvent');
			
		else
			Broadcastmessage(PName@"was found dead! Autopsy revealed it was a mush!", true, 'CriticalEvent');
	}
	
	else
	{
		if ( PTeam == 0 )
			Broadcastmessage(PName@"was found dead! Gib scan revealed the identity and that... he/she was a human!", true, 'CriticalEvent');
			
		else
			Broadcastmessage(PName@"was found dead! Gib scan revealed the identity and that... it was a mush!", true, 'CriticalEvent');
	}
}

simulated function OldTellTeam(byte PTeam, string PName)
{
	if ( PTeam == 0 )
		Broadcastmessage(PName@"died and is now out of the game! He/she was a human!", true, 'CriticalEvent');
		
	else
		Broadcastmessage(PName@"died and is now out of the game! It was a mush!", true, 'CriticalEvent');
}

simulated function MutatorScoreKill(Pawn Killer, Pawn Other, optional bool bTell)
{
	local Inventory I, prev;
	local bool b;
	local Pawn P;
	
	bTell = true;
	
	if ( ( Other.FindInventoryType(class'GusPackII.BeaconList') != None || Other.PlayerReplicationInfo.Team == 1 ) && Killer.FindInventoryType(class'SpottedMush') == None )
	{
		for ( P = Level.PawnList; P != None; P = P.NextPawn )
			if ( P.bIsPlayer && P.CanSee(Killer) && P.LineOfSightTo(Other) && P.PlayerReplicationInfo.Team == 0 && P != Killer )
			{
				for ( I = Killer.inventory; I != None; I = I.inventory )
					if ( HateList(I) != None || BeaconList(I) != None )
					{
						if ( BeaconList(I) != None ) b = True;
						
                        for ( prev = Killer.inventory; prev != None && prev.inventory != I; prev = prev.inventory );
                        
                        if ( prev != None )
                            prev.inventory = I.inventory;
                        
						I.Destroy();
					}
					
				if ( b )
				{
					BroadcastMessage(Killer.PlayerReplicationInfo.PlayerName$"'s name was cleared by"@P.PlayerReplicationInfo.PlayerName@"after killing a mush,"@Other.PlayerReplicationInfo.PlayerName$"!", true, 'CriticalEvent');
					
					break;
				}
			}
	}
	
	if ( !b && Other.PlayerReplicationInfo.Team == 0 ) CheckSuspects(Killer, Other);
	
	if ( bTell ) OldTellTeam(Other.PlayerReplicationInfo.Team, Other.PlayerReplicationInfo.PlayerName);
}

defaultproperties
{
}
