#exec AUDIO IMPORT FILE="Sounds\mm_humancert.wav" NAME="HumanCertified" GROUP="MushMatch"

class PlayerMonitor extends Actor;


var bool bHuman, bIsDead, bKnown, bGibbed;
var Pawn Instigator;
var InventorySpot Investigation;
var UTHumanCarcass MyCarcass;
var AttachInfo Monitoring;
var string OwnerName;
var byte OwnerTeam;

simulated function PostBeginPlay()
{
	if ( Pawn(Owner) == None || !CheckMonitorable(Pawn(Owner)) )
	{
		Warn(self@"has no owner or an unqualified one:"@Owner);
		Destroy();
	
		return;
	}
	
	SetCollisionSize(Owner.CollisionRadius * 0.5, Owner.CollisionHeight * 0.75);
	
	Monitoring = class'AttachInfo'.static.CreateAttachment(Owner, self, 16); // too bad it often gets stuck
	
	OwnerName = Pawn(Owner).PlayerReplicationInfo.PlayerName;
	OwnerTeam = Pawn(Owner).PlayerReplicationInfo.Team;
	
	ZoneChange(Region.Zone);
}

simulated function Tick(float TimeDelta)
{
	Super.Tick(TimeDelta);
	
	if ( bIsDead )
	{
		if ( bKnown )
			return;
	
		if ( MyCarcass == None )
		{
			if ( !bGibbed )
				SetTimer(0.2, true);
			
			if ( MyCarcass == None && !IsInState('PendingCarcass') ) GoToState('PendingCarcass');
		}
			
		else
			SetCollisionSize(Max(MyCarcass.CollisionRadius * 1.5, CollisionRadius * 1.8), CollisionHeight);
	
		return;
	}
		
	else if ( Pawn(Owner).PlayerReplicationInfo.Deaths > 0 && MushMatch(Level.Game).bSelected )
	{
		bIsDead = true;
		
		return;
	}
	
	if ( Pawn(Owner) == None )
	{
		Destroy();
	
		return;
	}
}

simulated function bool CheckMonitorable(Pawn Other)
{
	return ( Other.bIsPlayer && Other.PlayerReplicationInfo != None && ( mushmatch(Level.Game) == None || !mushmatch(Level.Game).bSelected || Other.PlayerReplicationInfo.Deaths <= 0 ) );
}

simulated function ZoneChange(ZoneInfo NewZone)
{
	if ( !CheckMonitorable(Pawn(Owner)) )
		return;

	if ( Pawn(Owner).PlayerReplicationInfo.Team == 1 )
	{
		if ( NewZone.bWaterZone )
			SetTimer(0.5, true);
			
		else
			SetTimer(0.0, false);
	}
	
	else if ( Pawn(Owner).PlayerReplicationInfo.Team == 0 && Pawn(Owner).FindInventoryType(class'NotMush') == none && NewZone.bWaterZone && !bHuman && (!NewZone.bPainZone) )
	{
		SetTimer(0.5, true);
	
		Timer();
	}
	
	else
		SetTimer(0.0, false);
}

static simulated function bool KnownDead(Pawn Other)
{
	local UTHumanCarcass C;
	local PlayerMonitor PM;

	foreach Other.ChildActors(class'PlayerMonitor', PM)
		if ( PM.bIsDead && PM.bKnown ) return true;
			
	return false;
}

simulated function Touch(Actor Other)
{
	local UTHumanCarcass C;

	if ( Pawn(Owner).PlayerReplicationInfo.Deaths <= 0 || bKnown || MyCarcass == None )
		return;
	
	if ( Pawn(Other) != None && Pawn(Other).bIsPlayer && Pawn(Other).PlayerReplicationInfo != None && ( Other == Instigator || IsInState('OldCarcass') ) )
	{
		if ( Pawn(Other).PlayerReplicationInfo.Team == 0 )
		{
			/*
			if ( bGibbed )
				MushMatchMutator(Level.Game.BaseMutator).MutatorScoreKill(Instigator, Pawn(Owner));
				
			else
				MushMatchMutator(Level.Game.BaseMutator).TellTeam(OwnerTeam, OwnerName);
			*/
				
			bKnown = true;
		}
		
		else if ( !bGibbed )
		{
			foreach AllActors(class'UTHumanCarcass', C) if ( C.PlayerOwner == Pawn(Owner).PlayerReplicationInfo && C.DrawType != DT_None )
				C.Destroy();
				
			Investigation.Destroy();
			Monitoring.Destroy();
			Destroy();
			
			return;
		}
	}
}

simulated function bool EncroachingOn(Actor Other)
{
	return false;
}

simulated function HitWall(Vector HitNormal, Actor HitWall)
{
	if ( (Velocity dot Velocity) < 256 ) Investigation = Spawn(class'InventorySpot', self,, MyCarcass.Location);
}

simulated function Timer()
{
	local Pawn p;
	local UTHumanCarcass C;
	
	if ( bIsDead )
	{
		foreach AllActors(class'UTHumanCarcass', C) if ( C.PlayerOwner == Pawn(Owner).PlayerReplicationInfo && C.DrawType != DT_None ) MyCarcass = C;
		
		return;
	}

	if ( !bHuman )
	{
		for ( p = Level.PawnList; p != None; p = p.NextPawn )
			if ( p.bIsPlayer && p != Pawn(Owner) && p.PlayerReplicationInfo != none && p.PlayerReplicationInfo.Deaths <= 0 && p.CanSee(Pawn(Owner)) && p.PlayerReplicationInfo.Team == 0 && Pawn(Owner).PlayerReplicationInfo.Team == 0 )
			{
				Pawn(Owner).AddInventory(Spawn(class'NotMush'));
			
				Broadcastmessage(Pawn(Owner).PlayerReplicationInfo.Playername@"was confirmed human by"@p.PlayerReplicationInfo.PlayerName$"!", true, 'CriticalEvent');
				
				SetTimer(0.0, false);
				bHuman = true;
                
                if ( PlayerPawn(Owner) != None )
                    Pawn(Owner).PlayOwnedSound(sound'GusPackII.MushMatch.HumanCertified');
			}
			
			// else Log( p @ Pawn(Owner) @ p.bIsPlayer @ p != Pawn(Owner) @ p.PlayerReplicationInfo != none @ p.PlayerReplicationInfo.Deaths <= 0 @ p.CanSee(Pawn(Owner)) @ p.PlayerReplicationInfo.Team == 0 );
	}
	
	else if ( Pawn(Owner).PlayerReplicationInfo.Team == 1 )
	{
		Pawn(Owner).Health -= 5;
		Pawn(Owner).TakeDamage(5, Pawn(Owner), Owner.Location, vect(0,0,0), 'waterburn');
	}
}

auto state Idle {}

state PendingCarcass
{
Begin:
	Sleep(5.0);
	
	if ( MyCarcass == None )
		bKnown = true;
		
	else
		GoToState('FreshCarcass');
}

state FreshCarcass
{
Begin:
	Sleep(5.0);
	
	GoToState('OldCarcass');
}

state OldCarcass {}

defaultproperties
{
     bCanTeleport=True
     Physics=PHYS_Projectile
     DrawType=DT_None
     bCollideActors=True
     bCollideWorld=True
}
