/*
 * Shoots "spore spits" that mushify
 * other players (and bots).
 */
 
#exec mesh import mesh=Sporifier anivfile=Models\Sporifier_a.3d datafile=Models\Sporifier_d.3d x=0 y=0 z=0 mlod=1

#exec mesh sequence mesh=Sporifier seq=All startframe=0 numframes=30 rate=20
#exec mesh sequence mesh=Sporifier seq=Still startframe=0 numframes=1 rate=1
#exec mesh sequence mesh=Sporifier seq=Fire startframe=0 numframes=12 rate=20
#exec mesh sequence mesh=Sporifier seq=Down startframe=13 numframes=9 rate=20
#exec mesh sequence mesh=Sporifier seq=Select startframe=22 numframes=8 rate=20

#exec meshmap new meshmap=Sporifier mesh=Sporifier
#exec meshmap scale meshmap=Sporifier x=0.1 y=0.1 z=0.2

#exec texture import name=JSporifier file=Textures\JSporifier.pcx group=Skins
#exec meshmap settexture meshmap=Sporifier num=0 texture=JSporifier
 
class Sporifier extends TournamentWeapon;


var bool bDanger;

simulated function PostBeginPlay()
{
	bDanger = true;
	
	if ( MushMatch(Level.Game).bSelected )
		MushSelected();
}

simulated function MushSelected()
{
	SetTimer(3.0, false);
}

simulated function float RateSelf(out int bUseAltMode)
{
	local Pawn P;
	
	bUseAltMode = 0;
	
	for ( P = Level.PawnList; P != none; P = P.nextPawn )
		if ( Pawn(Owner).FindInventoryType(class'GusPackII.SpottedMush') == None && P.bIsPlayer && ( ( FastTrace(P.Location) && p.CanSee(Owner) && p.PlayerReplicationInfo.Team == 0 && FRand() > 0.625 ) || P.Enemy == Owner || Pawn(Owner).PlayerReplicationInfo.Team == 0 ) )
		{
			bDanger = true;
			SetTimer(2.0, false);
			
			return 0;
		}
		
		else if ( bDanger )
			return 0;
			
	return 9000000000;
}

simulated function Timer()
{
	if ( bDanger )
		bDanger = false;
		
	else
		Super.Timer();
}

simulated function PlayFiring()
{
	PlayAnim('Fire', 1.4, 0.1);
	PlayOwnedSound(FireSound, SLOT_Misc, 1.0);
}

simulated function PlayIdle()
{
	PlayAnim('Still', 0.35, 0.5);
}

simulated function Tick(float TimeDelta)
{
	local Pawn p;
	
	Super.Tick(TimeDelta);
		
	if ( Owner == None || Owner.IsInState('Dying') || ( MushMatch(Level.Game) == none && Pawn(Owner).PlayerReplicationInfo != none && Pawn(Owner).PlayerReplicationInfo.Deaths > 0 ) )
	{
		Destroy();
		return;
	}
	
	if ( Pawn(Owner).Weapon != self ) return;
	
	for ( p = Level.PawnList; p != none; p = p.nextPawn )
		if ( p.bIsPlayer && p != Owner && p.PlayerReplicationInfo != none && p.PlayerReplicationInfo.Deaths <= 0 && p.CanSee(Owner) && Pawn(Owner).PlayerReplicationInfo.Team == 1  && p.PlayerReplicationInfo.Team == 0 )
		mushmatch(Level.Game).Spotmush(Pawn(Owner), p);
}

simulated function Projectile ProjectileFire(class<projectile> ProjClass, float ProjSpeed, bool bWarn)
{
	local Projectile pj;
	
	FireOffset.X += 15;
	
	while ( pj == None )
	{
		FireOffset.X -= 15;
	
		if ( FireOffset.X < -20 )
			return None;
		
		pj = Super.ProjectileFire(ProjClass, ProjSpeed, bWarn);
	}
	
	FireOffset.X = Default.FireOffset.X;
	
	pj.Instigator = Pawn(Owner);
	
	return pj;
}

defaultproperties
{
     AmmoName=Class'GusPackII.SporeCanister'
     PickupAmmoCount=18
     FireOffset=(X=32.000000)
     ProjectileClass=Class'GusPackII.SporeProj'
     AltProjectileClass=Class'GusPackII.SporeProj'
     FireSound=Sound'Botpack.BioRifle.GelHit'
     SelectSound=Sound'Botpack.enforcer.Cocking'
     DeathMessage="%o somehow died. And %k promised the mush would fix his defects. Huh."
     InventoryGroup=2
     PickupMessage="You got the Sporifier."
     ItemName="Sporifier"
     PlayerViewOffset=(X=22.100000,Y=-14.000000,Z=-17.000000)
     PlayerViewMesh=LodMesh'GusPackII.Sporifier'
     PickupViewMesh=LodMesh'GusPackII.Sporifier'
     ThirdPersonMesh=LodMesh'GusPackII.Sporifier'
     StatusIcon=Texture'Botpack.Icons.UsePulse'
     MaxDesireability=0.000000
     PickupSound=Sound'Botpack.enforcer.Cocking'
     Icon=Texture'Botpack.Icons.IconPulse'
     Mesh=LodMesh'GusPackII.Sporifier'
     Mass=65.000000
}
