//=============================================================================
// StartExpander.
//=============================================================================
class StartExpander expands CodeEvent;

var()	bool						bExpandOnInit, bLogTraces;
var()	float						StartInterval, TraceLift;

function Actor ClosestActor(Vector Begin, optional class<Actor> Type, optional out float Dist)
{
	local Actor A;
	local Actor Closest;

	if ( Type == None )
		Type = Class;

	foreach AllActors(Type, A)
		if ( Closest == None || VSize(A.Location - Begin) < VSize(Closest.Location - Begin) )
			Closest = A;

	Dist = VSize(Closest.Location - Location);

	return Closest;
}

function STResult SphereTrace(float Precision, bool bSelfOrigin, optional Vector Origin, optional float Distance)
{
	local vector v;
	local int i;
	local string HitActors;
	local STResult Current;
	local STResult Root;

	if ( Distance <= 0 )
		Distance = 4096;

	if ( bSelfOrigin )
		Origin = Location;

	for ( v.X = -1; v.X <= 1; v.X += Precision )
		for ( v.Y = -1; v.Y <= 1; v.Y += Precision )
			for ( v.Z = -1; v.Z <= 1; v.Z += Precision )
			{
				if ( Current == None )
				{
					Current = Spawn(class'STResult', self);

					Current.Root = Current;
				}

				else
				{
					Current.Next = Spawn(class'STResult', self);

					Current.Next.Root = Current.Root;
					Current.Next.Previous = Current;
					Current = Current.Next;
				}

				Current.Other = Trace(Current.TraceDest, Current.TraceNormal, v * Distance, Origin);
				Current.TraceDist = VSize(Current.TraceDest - Origin);

				Current.SetUp();
				i++;
			}

	Root = Current.Root;

	Log("Traced a sphere with"@i@"traces!");

	return Root;
}

function vector NewVector(float x, float y, float z)
{
	local Vector v;

	v.x = x;
	v.y = y;
	v.z = z;

	return v;
}

function ExpandPlayerStarts(class<Actor> Type, class<Actor> ExpansionType, optional float TraceLift)
{
	local int i, Successes;
	local Actor PS, ES;
	local Vector v, w;
	local STResult Current;

	foreach AllActors(Type, PS)
	{
		if ( ClassIsChildOf(PS.class, ExpansionType) )
			continue;

		Log("Attempting to expand"@PS);

		for ( Current = SphereTrace(StartInterval, false, PS.Location + NewVector(0, 0, TraceLift)); Current != None; Current = Current.Next )
		{
			if ( Current.Other == None || LevelInfo(Current.Other) != None || ( !FastTrace(Current.Other.Location + vect(0,0,-32), Current.Other.Location) && VSize(ClosestActor(Current.Other.Location, class'PlayerStart').Location - Current.Other.Location) > 48 ) && !FastTrace(Current.TraceDest + vect(0,0,-4096), Current.TraceDest) || Current.Other == None )
			{
				Trace(v, w, Current.TraceDest + vect(0,0,-4096), Current.TraceDest);

				Log("Spawning start at:"@v);
				ES = Spawn(ExpansionType,,, v);
				
				if ( PlayerStart(ES) != None && PlayerStart(PS) != None )
					PlayerStart(ES).TeamNumber = PlayerStart(PS).TeamNumber;

				Successes++;
			}

			Current.Destroy();
		}
	}

	Log("Succesfully spawned"@Successes@"new starts!");
}

function PreBeginPlay()
{	
	local int i;

	if ( bExpandOnInit )
		ExpandPlayerStarts(class'PlayerStart', class'ExpansionPlayerStart', TraceLift);
}

defaultproperties
{
     bExpandOnInit=True
}
