class FloatList extends Actor;

var FloatList next;
var byte allocated;
var float data[63];
var bool bUsed;

static function FloatList Concatenate(FloatList A, FloatList B)
{
    local int cpos;
    local FloatList cur, NA;
    
    NA = A.Duplicate();
    NA.Extend(B);
    
    return NA;
}

function Extend(FloatList Other)
{
    local int psource, pdest;
    local FloatList csource, cdest;
    
    if ( next != None )
        for ( cdest = next; cdest.next != None; cdest = cdest.next );
        
    else
        cdest = self;
    
    pdest = cdest.allocated;
    csource = Other;
    
    while ( psource < csource.allocated )
    {
        cdest.data[pdest] = csource.data[psource];
    
        psource++;
        pdest++;
        
        if ( psource == 63 )
        {
            if ( csource.next == None )
                return;
                
            csource = csource.next;
            psource = 0;
        }
        
        if ( pdest == 63 )
        {
            if ( cdest.next == None )
                cdest.next = Spawn(class'FloatList');
                
            cdest = cdest.next;
            pdest = 0;
        }
    }
}

function FloatList Duplicate()
{
    local FloatList first, cur, cursrc;
    local int cpos;
    
    first = Spawn(class'FloatList');
    cur = first;
    cursrc = self;
    
    while ( True )
    {
        for ( cpos = 0; cpos < cursrc.allocated; cpos++ )
            cur.data[cpos] = cursrc.data[cpos];
            
        cur.allocated = cursrc.allocated;
        
        cursrc = cursrc.next;
        
        if ( cursrc == None )
            break;
            
        cur.next = Spawn(class'FloatList');
        cur = cur.next;
    }
    
    return first;
}

function int Length()
{
    local int res;
    local FloatList fl;
    
    for ( fl = self; fl != None; fl = fl.next )
        res += fl.allocated;
        
    return res;
}

function AddFloat(float f)
{
    local FloatList last;

    if ( allocated == 63 )
    {
        for ( last = self; last.next != None; last = last.next );
    
        if ( last == self )
        {
            next = Spawn(class'SectorList.FloatList');
            last = next;
        }
            
        return;
    }
    
    data[allocated++] = f;
    return;
}

function LeftShift(int pos)
{
    local FloatList sec;
    
    for (sec = self; sec != None && pos > 62; sec = sec.next ) pos -= 63;
    
    if ( sec == None )
        return;
    
    while ( sec != None )
    {
        while ( pos < sec.allocated - 1 ) sec.data[pos] = sec.data[++pos];
        
        if ( sec.allocated == 63 )
        {
            if ( sec.next != None )
                sec.data[pos] = sec.next.data[0];        
            
            else
            {
                sec.data[pos] = 0;
                sec.allocated--;
                
                break;
            }
                
            pos = 0;
            sec = sec.next;
        }
            
        else
        {
            data[pos] = 0; // deinit
            sec.allocated--;
            
            if ( sec.allocated == 0 )
                sec.Destroy();
            
            break;
        }
    }
}

function bool RemoveFloat(int position)
{
    local FloatList sect;

    if ( position > 62 )
    {
        for ( sect = self; sect != None && position > 62; sect = sect.next ) position -= 63;
        
        if ( sect == None )
            return false;
        
        return sect.RemoveFloat(position);
    }
    
    if ( position < allocated )
        return false;
        
    LeftShift(position);
    return true;
}

function OptionalFloat Get(int position)
{
    local FloatList sect;

    if ( position > 62 )
    {
        for ( sect = self; sect != None && position > 62; sect = sect.next ) position -= 63;
        
        if ( sect == None )
            return class'OptionalFloat'.static.Empty(self);
        
        return class'OptionalFloat'.static.Of(self, sect.data[position]);
    }
    
    return class'OptionalFloat'.static.Of(self, data[position]);
}

function AddRepeated(float other, int repeats)
{
    if ( repeats < 1 )
        return;
        
    while ( repeats > 1 )
    {
        AddFloat(other);
        repeats--;
    }
    
    return;
}

function int Repeat(int repeats)
{
    while ( repeats > 0 )
    {
        Extend(self);
        repeats--;
    }
}

function string Dump()
{
    local int i;
    local string r;
    local FloatList cur;

    r = "[";
    cur = self;
    
    while ( i < allocated )
    {
        r = r $ cur.data[i];
     
        if ( i == allocated )
        {
            if ( next != None )
            {
                cur = next;
                i = 0;
            }
            
            else
                break;
        }
    
        else
            i++;
            
        if ( i < allocated )
            r = r $ ", ";
    }
    
    r = r $ "]";
    return r;
}

defaultproperties
{
    bHidden=True
}